@@ -17,13 +17,14 @@
 #
 from __future__ import absolute_import, division, print_function
 
+
 __metaclass__ = type
 DOCUMENTATION = """
 module: ios_facts
 author:
 - Peter Sprygada (@privateip)
 - Sumit Jaiswal (@justjais)
-short_description: Collect facts from remote devices running Cisco IOS
+short_description: Module to collect facts from remote devices.
 description:
 - Collects a base set of device facts from a remote device that is running IOS.  This
   module prepends all of the base network fact keys with C(ansible_net_<fact>).  The
@@ -33,7 +34,7 @@ version_added: 1.0.0
 extends_documentation_fragment:
 - cisco.ios.ios
 notes:
-  - Tested against IOS 15.6
+  - Tested against Cisco IOSXE Version 17.3 on CML and IOS 15.6 for L2 specific resource.
   - Facts gathering for L3 devices are supposed to produce blank output for unsupported
     resources like vlan.
   - This module works with connection C(network_cli).
@@ -47,7 +48,7 @@ options:
     - Specify a list of values to include a larger subset.
     - Use a value with an initial C(!) to collect all facts except that subset.
     required: false
-    default: '!config'
+    default: 'min'
     type: list
     elements: str
   gather_network_resources:
@@ -56,9 +57,11 @@ options:
       Possible values for this argument include all and the resources like interfaces,
       vlans etc. Can specify a list of values to include a larger subset. Values can
       also be used with an initial C(!) to specify that a specific subset should
-      not be collected. Valid subsets are 'all', 'interfaces', 'l2_interfaces', 'vlans',
-      'lag_interfaces', 'lacp', 'lacp_interfaces', 'lldp_global', 'lldp_interfaces',
-      'l3_interfaces', 'acl_interfaces', 'static_routes', 'acls'.
+      not be collected. Valid subsets are 'bgp_global', 'l3_interfaces', 'lag_interfaces',
+      'ntp_global', 'acls', 'hostname', 'interfaces', 'lldp_interfaces', 'logging_global',
+      'ospf_interfaces', 'ospfv2', 'prefix_lists', 'static_routes', 'acl_interfaces',
+      'all', 'bgp_address_family', 'l2_interfaces', 'lacp', 'lacp_interfaces', 'lldp_global',
+      'ospfv3', 'snmp_server', 'vlans'.
     type: list
     elements: str
   available_network_resources:
@@ -207,15 +210,13 @@ ansible_net_neighbors:
   type: dict
 """
 from ansible.module_utils.basic import AnsibleModule
+
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.facts.facts import (
     FactsArgs,
 )
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import (
-    Facts,
     FACT_RESOURCE_SUBSETS,
-)
-from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
-    ios_argument_spec,
+    Facts,
 )
 
 
@@ -226,21 +227,12 @@ def main():
     :returns: ansible_facts
     """
     argument_spec = FactsArgs.argument_spec
-    argument_spec.update(ios_argument_spec)
-    module = AnsibleModule(
-        argument_spec=argument_spec, supports_check_mode=True
-    )
+    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
     warnings = []
-    if module.params["gather_subset"] == "!config":
-        warnings.append(
-            "default value for `gather_subset` will be changed to `min` from `!config` v2.11 onwards"
-        )
 
     ansible_facts = {}
     if module.params.get("available_network_resources"):
-        ansible_facts["available_network_resources"] = sorted(
-            FACT_RESOURCE_SUBSETS.keys()
-        )
+        ansible_facts["available_network_resources"] = sorted(FACT_RESOURCE_SUBSETS.keys())
     result = Facts(module).get_facts()
     additional_facts, additional_warnings = result
     ansible_facts.update(additional_facts)
