@@ -5,14 +5,15 @@
 
 from __future__ import absolute_import, division, print_function
 
+
 __metaclass__ = type
 
 from textwrap import dedent
-from ansible_collections.cisco.ios.tests.unit.compat.mock import patch
+
 from ansible_collections.cisco.ios.plugins.modules import ios_l3_interfaces
-from ansible_collections.cisco.ios.tests.unit.modules.utils import (
-    set_module_args,
-)
+from ansible_collections.cisco.ios.tests.unit.compat.mock import patch
+from ansible_collections.cisco.ios.tests.unit.modules.utils import set_module_args
+
 from .ios_module import TestIosModule
 
 
@@ -23,39 +24,35 @@ class TestIosL3InterfacesModule(TestIosModule):
         super(TestIosL3InterfacesModule, self).setUp()
 
         self.mock_get_config = patch(
-            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
+            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config",
         )
         self.get_config = self.mock_get_config.start()
 
         self.mock_load_config = patch(
-            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
+            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config",
         )
         self.load_config = self.mock_load_config.start()
 
         self.mock_get_resource_connection_config = patch(
             "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base."
-            "get_resource_connection"
-        )
-        self.get_resource_connection_config = (
-            self.mock_get_resource_connection_config.start()
+            "get_resource_connection",
         )
+        self.get_resource_connection_config = self.mock_get_resource_connection_config.start()
 
         self.mock_get_resource_connection_facts = patch(
             "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
-            "get_resource_connection"
-        )
-        self.get_resource_connection_facts = (
-            self.mock_get_resource_connection_facts.start()
+            "get_resource_connection",
         )
+        self.get_resource_connection_facts = self.mock_get_resource_connection_facts.start()
 
         self.mock_edit_config = patch(
-            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers.CliProvider.edit_config"
+            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers.CliProvider.edit_config",
         )
         self.edit_config = self.mock_edit_config.start()
 
         self.mock_execute_show_command = patch(
             "ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.l3_interfaces.l3_interfaces."
-            "L3_InterfacesFacts.get_l3_interfaces_data"
+            "L3_InterfacesFacts.get_l3_interfaces_data",
         )
         self.execute_show_command = self.mock_execute_show_command.start()
 
@@ -77,7 +74,7 @@ class TestIosL3InterfacesModule(TestIosModule):
             interface Serial3/0
              ipv6 address fd5d:12c9:2201:1::1/64
             interface Serial7/0
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -89,12 +86,12 @@ class TestIosL3InterfacesModule(TestIosModule):
                     dict(
                         name="Serial3/0",
                         ipv6=[
-                            dict(address="FD5D:12C9:2201:1::1/64", cga=True)
+                            dict(address="FD5D:12C9:2201:1::1/64", cga=True),
                         ],
                     ),
                 ],
                 state="merged",
-            )
+            ),
         )
         result = self.execute_module(changed=False)
         self.assertEqual(result["commands"], [])
@@ -118,7 +115,7 @@ class TestIosL3InterfacesModule(TestIosModule):
              ip address 192.168.0.3 255.255.255.0
             interface Serial1/0
              description Configured by PAUL
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -132,11 +129,12 @@ class TestIosL3InterfacesModule(TestIosModule):
                         ipv4=[dict(address="192.168.0.2/24")],
                     ),
                     dict(
-                        name="Serial1/0", ipv4=[dict(address="192.168.0.3/24")]
+                        name="Serial1/0",
+                        ipv4=[dict(address="192.168.0.3/24")],
                     ),
                 ],
                 state="overridden",
-            )
+            ),
         )
         commands = [
             "interface GigabitEthernet0/3",
@@ -172,7 +170,7 @@ class TestIosL3InterfacesModule(TestIosModule):
              ip address 192.168.0.3 255.255.255.0
             interface Serial1/0
              description Configured by PAUL
-            """
+            """,
         )
         set_module_args(dict(state="deleted"))
         commands = [
@@ -204,7 +202,7 @@ class TestIosL3InterfacesModule(TestIosModule):
              ip address 192.168.0.3 255.255.255.0
             interface Serial1/0
              description Configured by PAUL
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -218,7 +216,8 @@ class TestIosL3InterfacesModule(TestIosModule):
                         ipv4=[dict(address="192.168.0.2/24")],
                     ),
                     dict(
-                        name="Serial1/0", ipv4=[dict(address="192.168.0.5/24")]
+                        name="Serial1/0",
+                        ipv4=[dict(address="192.168.0.5/24")],
                     ),
                     dict(
                         name="GigabitEthernet0/3.100",
@@ -226,7 +225,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                     ),
                 ],
                 state="replaced",
-            )
+            ),
         )
         commands = [
             "interface GigabitEthernet0/3",
@@ -249,14 +248,14 @@ class TestIosL3InterfacesModule(TestIosModule):
             dict(
                 running_config="interface GigabitEthernet0/3.100\nencapsulation dot1Q 20\n ip address 192.168.0.3 255.255.255.0\n",
                 state="parsed",
-            )
+            ),
         )
         result = self.execute_module(changed=False)
         parsed_list = [
             {
                 "name": "GigabitEthernet0/3.100",
                 "ipv4": [{"address": "192.168.0.3/24"}],
-            }
+            },
         ]
         self.assertEqual(parsed_list, result["parsed"])
 
@@ -271,7 +270,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                                 address="dhcp",
                                 dhcp_client="GigabitEthernet0/3",
                                 dhcp_hostname="abc.com",
-                            )
+                            ),
                         ],
                         ipv6=[dict(address="FD5D:12C9:2202:1::1/64")],
                     ),
@@ -284,11 +283,12 @@ class TestIosL3InterfacesModule(TestIosModule):
                         ipv4=[dict(address="192.168.0.4/24", secondary=True)],
                     ),
                     dict(
-                        name="Serial1/0", ipv4=[dict(address="192.168.0.5/24")]
+                        name="Serial1/0",
+                        ipv4=[dict(address="192.168.0.5/24")],
                     ),
                 ],
                 state="rendered",
-            )
+            ),
         )
         commands = [
             "interface GigabitEthernet0/3",
@@ -324,7 +324,7 @@ class TestIosL3InterfacesModule(TestIosModule):
              ip address 192.168.0.3 255.255.255.0
             interface Serial1/0
              description Configured by PAUL
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -336,8 +336,8 @@ class TestIosL3InterfacesModule(TestIosModule):
                                 dhcp=dict(
                                     client_id="GigabitEthernet0/2",
                                     hostname="test.com",
-                                )
-                            )
+                                ),
+                            ),
                         ],
                     ),
                     dict(
@@ -356,20 +356,21 @@ class TestIosL3InterfacesModule(TestIosModule):
                         name="Serial3/0",
                         ipv6=[
                             dict(
-                                address="FD5D:12C9:2201:1::1/64", anycast=True
-                            )
+                                address="FD5D:12C9:2201:1::1/64",
+                                anycast=True,
+                            ),
                         ],
                     ),
                     dict(
                         name="Serial4/0",
                         ipv6=[
-                            dict(address="FD5D:12C9:2201:2::1/64", cga=True)
+                            dict(address="FD5D:12C9:2201:2::1/64", cga=True),
                         ],
                     ),
                     dict(
                         name="Serial5/0",
                         ipv6=[
-                            dict(address="FD5D:12C9:2201:3::1/64", eui=True)
+                            dict(address="FD5D:12C9:2201:3::1/64", eui=True),
                         ],
                     ),
                     dict(
@@ -378,7 +379,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:4::1/64",
                                 link_local=True,
-                            )
+                            ),
                         ],
                     ),
                     dict(
@@ -387,7 +388,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:5::1/64",
                                 segment_routing=dict(ipv6_sr=True),
-                            )
+                            ),
                         ],
                     ),
                     dict(
@@ -396,12 +397,12 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:6::1/64",
                                 segment_routing=dict(default=True),
-                            )
+                            ),
                         ],
                     ),
                 ],
                 state="merged",
-            )
+            ),
         )
         commands = [
             "interface GigabitEthernet0/1",
@@ -451,7 +452,7 @@ class TestIosL3InterfacesModule(TestIosModule):
              ipv6 address fd5d:12c9:2201:4::1/64 link-local
             interface Serial5/0
              ipv6 address fd5d:12c9:2201:3::1/64 eui
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -463,8 +464,8 @@ class TestIosL3InterfacesModule(TestIosModule):
                                 dhcp=dict(
                                     client_id="GigabitEthernet0/2",
                                     hostname="test.com",
-                                )
-                            )
+                                ),
+                            ),
                         ],
                     ),
                     dict(
@@ -479,20 +480,21 @@ class TestIosL3InterfacesModule(TestIosModule):
                         name="Serial3/0",
                         ipv6=[
                             dict(
-                                address="FD5D:12C9:2201:1::1/64", anycast=True
-                            )
+                                address="FD5D:12C9:2201:1::1/64",
+                                anycast=True,
+                            ),
                         ],
                     ),
                     dict(
                         name="Serial4/0",
                         ipv6=[
-                            dict(address="FD5D:12C9:2201:2::1/64", cga=True)
+                            dict(address="FD5D:12C9:2201:2::1/64", cga=True),
                         ],
                     ),
                     dict(
                         name="Serial5/0",
                         ipv6=[
-                            dict(address="FD5D:12C9:2201:3::1/64", eui=True)
+                            dict(address="FD5D:12C9:2201:3::1/64", eui=True),
                         ],
                     ),
                     dict(
@@ -501,7 +503,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:4::1/64",
                                 link_local=True,
-                            )
+                            ),
                         ],
                     ),
                     dict(
@@ -510,7 +512,7 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:5::1/64",
                                 segment_routing=dict(ipv6_sr=True),
-                            )
+                            ),
                         ],
                     ),
                     dict(
@@ -519,12 +521,12 @@ class TestIosL3InterfacesModule(TestIosModule):
                             dict(
                                 address="FD5D:12C9:2201:6::1/64",
                                 segment_routing=dict(default=True),
-                            )
+                            ),
                         ],
                     ),
                 ],
                 state="merged",
-            )
+            ),
         )
         commands = []
         result = self.execute_module(changed=False)
