@@ -19,14 +19,17 @@
 
 from __future__ import absolute_import, division, print_function
 
+
 __metaclass__ = type
 
 from textwrap import dedent
-from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
+
 from ansible_collections.cisco.nxos.plugins.modules import nxos_snmp_server
+from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
 
 from .nxos_module import TestNxosModule, set_module_args
 
+
 ignore_provider_arg = True
 
 
@@ -38,14 +41,12 @@ class TestNxosSnmpServerModule(TestNxosModule):
         super(TestNxosSnmpServerModule, self).setUp()
 
         self.mock_get_resource_connection = patch(
-            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base.get_resource_connection"
-        )
-        self.get_resource_connection = (
-            self.mock_get_resource_connection.start()
+            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base.get_resource_connection",
         )
+        self.get_resource_connection = self.mock_get_resource_connection.start()
 
         self.mock_get_config = patch(
-            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.snmp_server.snmp_server.Snmp_serverFacts.get_config"
+            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.snmp_server.snmp_server.Snmp_serverFacts.get_config",
         )
         self.get_config = self.mock_get_config.start()
 
@@ -58,7 +59,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
         # test merged for linear attributes
         self.get_config.return_value = dedent(
             """\
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -68,25 +69,17 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     context=dict(name="public", vrf="siteA", instance="test"),
                     counter=dict(cache=dict(timeout=1800)),
                     drop=dict(unknown_engine_id=True, unknown_user=True),
-                    engine_id=dict(
-                        local="'00:00:00:63:00:01:00:10:20:15:10:03'"
-                    ),
+                    engine_id=dict(local="'00:00:00:63:00:01:00:10:20:15:10:03'"),
                     communities=[
                         dict(name="private", group="network-admin"),
                         dict(community="public", use_ipv4acl="myacl"),
                     ],
                     global_enforce_priv=True,
                     location="lab",
-                    mib=dict(
-                        community_map=dict(
-                            community="public", context="public1"
-                        )
-                    ),
+                    mib=dict(community_map=dict(community="public", context="public1")),
                     packetsize=484,
                     protocol=dict(enable=True),
-                    source_interface=dict(
-                        informs="Ethernet1/1", traps="Ethernet1/2"
-                    ),
+                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                     system_shutdown=True,
                     tcp_session=dict(auth=True),
                 ),
@@ -137,7 +130,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server location lab
             snmp-server mib community-map public context public1
             snmp-server source-interface traps Ethernet1/2
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -147,21 +140,13 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     context=dict(name="public", vrf="siteA"),
                     counter=dict(cache=dict(timeout=1800)),
                     drop=dict(unknown_engine_id=True, unknown_user=True),
-                    engine_id=dict(
-                        local="00:00:00:63:00:01:00:10:20:15:10:03"
-                    ),
+                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                     global_enforce_priv=True,
                     location="lab",
-                    mib=dict(
-                        community_map=dict(
-                            community="public", context="public1"
-                        )
-                    ),
+                    mib=dict(community_map=dict(community="public", context="public1")),
                     packetsize=484,
                     protocol=dict(enable=True),
-                    source_interface=dict(
-                        informs="Ethernet1/1", traps="Ethernet1/2"
-                    ),
+                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                     system_shutdown=True,
                     tcp_session=dict(auth=True),
                 ),
@@ -192,7 +177,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server location lab
             snmp-server mib community-map public context public1
             snmp-server source-interface traps Ethernet1/2
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -201,21 +186,13 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     contact="testswitch@localhost",
                     context=dict(name="public", vrf="siteA"),
                     counter=dict(cache=dict(timeout=1860)),
-                    engine_id=dict(
-                        local="00:00:00:63:00:01:00:10:20:15:10:03"
-                    ),
+                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                     global_enforce_priv=True,
                     location="lab",
-                    mib=dict(
-                        community_map=dict(
-                            community="public", context="public1"
-                        )
-                    ),
+                    mib=dict(community_map=dict(community="public", context="public1")),
                     packetsize=484,
                     protocol=dict(enable=True),
-                    source_interface=dict(
-                        informs="Ethernet1/3", traps="Ethernet1/2"
-                    ),
+                    source_interface=dict(informs="Ethernet1/3", traps="Ethernet1/2"),
                     tcp_session=dict(auth=True),
                 ),
                 state="replaced",
@@ -253,7 +230,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server location lab
             snmp-server mib community-map public context public1
             snmp-server source-interface traps Ethernet1/2
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -262,21 +239,13 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     contact="testswitch@localhost",
                     context=dict(name="public", vrf="siteA"),
                     counter=dict(cache=dict(timeout=1860)),
-                    engine_id=dict(
-                        local="00:00:00:63:00:01:00:10:20:15:10:03"
-                    ),
+                    engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
                     global_enforce_priv=True,
                     location="lab",
-                    mib=dict(
-                        community_map=dict(
-                            community="public", context="public1"
-                        )
-                    ),
+                    mib=dict(community_map=dict(community="public", context="public1")),
                     packetsize=484,
                     protocol=dict(enable=True),
-                    source_interface=dict(
-                        informs="Ethernet1/3", traps="Ethernet1/2"
-                    ),
+                    source_interface=dict(informs="Ethernet1/3", traps="Ethernet1/2"),
                     tcp_session=dict(auth=True),
                 ),
                 state="overridden",
@@ -294,11 +263,36 @@ class TestNxosSnmpServerModule(TestNxosModule):
         result = self.execute_module(changed=True)
         self.assertEqual(set(result["commands"]), set(commands))
 
+    def test_nxos_snmp_server_location_spaces(self):
+        # test replaced for linear attributes
+        self.get_config.return_value = dedent(
+            """\
+            snmp-server contact testswitch@localhost
+            snmp-server location lab
+            """,
+        )
+        set_module_args(
+            dict(
+                config=dict(
+                    contact="test-switch @t localhost",
+                    location="long/and.(complicated) address",
+                ),
+                state="replaced",
+            ),
+            ignore_provider_arg,
+        )
+        commands = [
+            "snmp-server contact test-switch @t localhost",
+            "snmp-server location long/and.(complicated) address",
+        ]
+        result = self.execute_module(changed=True)
+        self.assertEqual(set(result["commands"]), set(commands))
+
     def test_nxos_snmp_server_traps_merged(self):
         # test merged for traps
         self.get_config.return_value = dedent(
             """\
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -307,7 +301,10 @@ class TestNxosSnmpServerModule(TestNxosModule):
                         aaa=dict(server_state_change=True),
                         bridge=dict(enable=True),
                         callhome=dict(event_notify=True, smtp_send_fail=True),
-                    )
+                        bgp=dict(enable=True),
+                        ospf=dict(enable=True),
+                        ospfv3=dict(enable=True),
+                    ),
                 ),
                 state="merged",
             ),
@@ -319,6 +316,9 @@ class TestNxosSnmpServerModule(TestNxosModule):
             "snmp-server enable traps bridge topologychange",
             "snmp-server enable traps callhome event-notify",
             "snmp-server enable traps callhome smtp-send-fail",
+            "snmp-server enable traps bgp",
+            "snmp-server enable traps ospf",
+            "snmp-server enable traps ospfv3",
         ]
         result = self.execute_module(changed=True)
         self.assertEqual(set(result["commands"]), set(commands))
@@ -332,7 +332,10 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server enable traps bridge topologychange
             snmp-server enable traps callhome event-notify
             snmp-server enable traps callhome smtp-send-fail
-            """
+            snmp-server enable traps bgp
+            snmp-server enable traps ospf
+            snmp-server enable traps ospfv3
+            """,
         )
         set_module_args(
             dict(
@@ -341,7 +344,10 @@ class TestNxosSnmpServerModule(TestNxosModule):
                         aaa=dict(server_state_change=True),
                         bridge=dict(enable=True),
                         callhome=dict(event_notify=True, smtp_send_fail=True),
-                    )
+                        bgp=dict(enable=True),
+                        ospf=dict(enable=True),
+                        ospfv3=dict(enable=True),
+                    ),
                 ),
                 state="merged",
             ),
@@ -360,7 +366,8 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server enable traps bridge topologychange
             snmp-server enable traps callhome event-notify
             snmp-server enable traps callhome smtp-send-fail
-            """
+            snmp-server enable traps link cisco-xcvr-mon-status-chg
+            """,
         )
         set_module_args(
             dict(
@@ -369,7 +376,8 @@ class TestNxosSnmpServerModule(TestNxosModule):
                         aaa=dict(server_state_change=True),
                         bridge=dict(enable=True),
                         cfs=dict(merge_failure=True),
-                    )
+                        link=dict(cisco_xcvr_mon_status_chg=False),
+                    ),
                 ),
                 state="replaced",
             ),
@@ -379,6 +387,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             "no snmp-server enable traps callhome event-notify",
             "no snmp-server enable traps callhome smtp-send-fail",
             "snmp-server enable traps cfs merge-failure",
+            "no snmp-server enable traps link cisco-xcvr-mon-status-chg",
         ]
         result = self.execute_module(changed=True)
         self.assertEqual(set(result["commands"]), set(commands))
@@ -392,7 +401,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server enable traps bridge topologychange
             snmp-server enable traps callhome event-notify
             snmp-server enable traps callhome smtp-send-fail
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -401,7 +410,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                         aaa=dict(server_state_change=True),
                         bridge=dict(enable=True),
                         cfs=dict(merge_failure=True),
-                    )
+                    ),
                 ),
                 state="replaced",
             ),
@@ -419,7 +428,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
         # test merged for hosts
         self.get_config.return_value = dedent(
             """\
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -431,9 +440,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                             community="public",
                             traps=True,
                         ),
-                        dict(
-                            host="192.168.1.1", source_interface="Ethernet1/1"
-                        ),
+                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                         dict(
                             host="192.168.2.1",
                             version="1",
@@ -459,7 +466,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                             priv="private",
                             informs=True,
                         ),
-                    ]
+                    ],
                 ),
                 state="merged",
             ),
@@ -486,7 +493,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server host 192.168.1.1 source-interface Ethernet1/1
             snmp-server host 192.168.2.1 traps version 1 private
             snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -498,9 +505,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                             community="public",
                             traps=True,
                         ),
-                        dict(
-                            host="192.168.1.1", source_interface="Ethernet1/1"
-                        ),
+                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                         dict(
                             host="192.168.2.1",
                             version="1",
@@ -526,7 +531,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                             priv="private",
                             informs=True,
                         ),
-                    ]
+                    ],
                 ),
                 state="merged",
             ),
@@ -545,7 +550,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server host 192.168.1.1 source-interface Ethernet1/1
             snmp-server host 192.168.2.1 traps version 1 private
             snmp-server host 192.168.3.1 informs version 3 auth private udp-port 65550
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -557,9 +562,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                             community="public",
                             traps=True,
                         ),
-                        dict(
-                            host="192.168.1.1", source_interface="Ethernet1/1"
-                        ),
+                        dict(host="192.168.1.1", source_interface="Ethernet1/1"),
                         dict(
                             host="192.168.2.1",
                             version="1",
@@ -574,7 +577,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
                         ),
                         dict(host="192.168.2.1", filter_vrf="siteA"),
                         dict(host="192.168.4.1", use_vrf="siteB"),
-                    ]
+                    ],
                 ),
                 state="replaced",
             ),
@@ -593,7 +596,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
         # test merged for users
         self.get_config.return_value = dedent(
             """\
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -618,9 +621,9 @@ class TestNxosSnmpServerModule(TestNxosModule):
                                     password="0x5632724fb8ac3699296af26281e1d0f1",
                                     engine_id="2:2:2:2:2",
                                     priv=dict(
-                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1"
+                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1",
                                     ),
-                                    localized_key=True,
+                                    localizedv2_key=True,
                                 ),
                             ),
                             dict(
@@ -637,8 +640,22 @@ class TestNxosSnmpServerModule(TestNxosModule):
                                     localized_key=True,
                                 ),
                             ),
-                        ]
-                    )
+                            dict(
+                                user="snmp_user_4",
+                                group="network-admin",
+                                authentication=dict(
+                                    algorithm="sha-256",
+                                    password="0x5632724fb8ac3699296af26281e1d0f1",
+                                    engine_id="4:4:4:4:4",
+                                    priv=dict(
+                                        privacy_password="0x5632724fb8ac3699296af26281e1d0f1",
+                                        aes_128=True,
+                                    ),
+                                    localized_key=True,
+                                ),
+                            ),
+                        ],
+                    ),
                 ),
                 state="merged",
             ),
@@ -648,9 +665,11 @@ class TestNxosSnmpServerModule(TestNxosModule):
             "snmp-server user snmp_user_2 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1 priv 0x5632724fb8ac3699296af26281e1d0f1"
             " localizedkey engineID 2:2:2:2:2",
             "snmp-server user snmp_user_3 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1 priv aes-128"
-            " 0x5632724fb8ac3699296af26281e1d0f1 localizedkey engineID 3:3:3:3:3",
+            " 0x5632724fb8ac3699296af26281e1d0f1 localizedV2key engineID 3:3:3:3:3",
             "snmp-server user snmp_user_1 network-admin auth md5 0x5632724fb8ac3699296af26281e1d0f1"
             " localizedkey engineID 1:1:1:1:1",
+            "snmp-server user snmp_user_4 network-admin auth sha-256 0x5632724fb8ac3699296af26281e1d0f1 priv aes-128"
+            " 0x5632724fb8ac3699296af26281e1d0f1 localizedkey engineID 4:4:4:4:4",
         ]
         result = self.execute_module(changed=True)
         self.assertEqual(set(result["commands"]), set(commands))
@@ -659,10 +678,11 @@ class TestNxosSnmpServerModule(TestNxosModule):
         # test merged for users
         self.get_config.return_value = dedent(
             """\
-            snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv 0x5632724fb8ac3699296af262 localizedkey engineID 2:2:2:2:2
+            snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv 0x5632724fb8ac3699296af262 localizedV2key engineID 2:2:2:2:2
             snmp-server user user3 network-admin auth md5 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3
             snmp-server user user1 network-admin auth md5 0x5632724fb8ac3699296af262 localizedkey engineID 1:1:1:1:1
-            """
+            snmp-server user user4 network-admin auth sha-256 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 4:4:4:4:4
+            """,
         )
         set_module_args(
             dict(
@@ -686,10 +706,8 @@ class TestNxosSnmpServerModule(TestNxosModule):
                                     algorithm="md5",
                                     password="0x5632724fb8ac3699296af262",
                                     engine_id="2:2:2:2:2",
-                                    priv=dict(
-                                        privacy_password="0x5632724fb8ac3699296af262"
-                                    ),
-                                    localized_key=True,
+                                    priv=dict(privacy_password="0x5632724fb8ac3699296af262"),
+                                    localizedv2_key=True,
                                 ),
                             ),
                             dict(
@@ -706,8 +724,22 @@ class TestNxosSnmpServerModule(TestNxosModule):
                                     localized_key=True,
                                 ),
                             ),
-                        ]
-                    )
+                            dict(
+                                user="user4",
+                                group="network-admin",
+                                authentication=dict(
+                                    algorithm="sha-256",
+                                    password="0x5632724fb8ac3699296af262",
+                                    engine_id="4:4:4:4:4",
+                                    priv=dict(
+                                        privacy_password="0x5632724fb8ac3699296af262",
+                                        aes_128=True,
+                                    ),
+                                    localized_key=True,
+                                ),
+                            ),
+                        ],
+                    ),
                 ),
                 state="merged",
             ),
@@ -723,7 +755,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server user user2 network-admin auth md5 0x5632724fb8ac3699296af262 priv 0x5632724fb8ac3699296af262 localizedkey engineID 2:2:2:2:2
             snmp-server user user3 network-admin auth md5 0x5632724fb8ac3699296af262 priv aes-128 0x5632724fb8ac3699296af262 localizedkey engineID 3:3:3:3:3
             snmp-server user user1 network-admin auth md5 0x5632724fb8ac3699296af262 localizedkey engineID 1:1:1:1:1
-            """
+            """,
         )
         set_module_args(
             dict(
@@ -754,8 +786,8 @@ class TestNxosSnmpServerModule(TestNxosModule):
                                     localized_key=True,
                                 ),
                             ),
-                        ]
-                    )
+                        ],
+                    ),
                 ),
                 state="replaced",
             ),
@@ -792,7 +824,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server location lab
             snmp-server mib community-map public context public1
             snmp-server source-interface traps Ethernet1/2
-            """
+            """,
         )
         set_module_args(dict(state="deleted"), ignore_provider_arg)
         commands = [
@@ -840,7 +872,9 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     snmp-server location lab
                     snmp-server mib community-map public context public1
                     snmp-server source-interface traps Ethernet1/2
-                    """
+                    snmp-server user 1234 network-admin auth md5 0x7d425fbf09417c44bca69e1d9e9ce889 priv 0x7d425fbf09417c44bca69e1d9e9ce889 localizedkey
+                    snmp-server user snmp_user_1 network-operator auth md5 0x5632724fb8ac3699296af26281e1d0f1 localizedkey
+                    """,
                 ),
                 state="parsed",
             ),
@@ -860,14 +894,35 @@ class TestNxosSnmpServerModule(TestNxosModule):
             engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
             global_enforce_priv=True,
             location="lab",
-            mib=dict(
-                community_map=dict(community="public", context="public1")
-            ),
+            mib=dict(community_map=dict(community="public", context="public1")),
             packetsize=484,
             protocol=dict(enable=True),
             source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
             system_shutdown=True,
             tcp_session=dict(auth=True),
+            users=dict(
+                auth=[
+                    dict(
+                        user="1234",
+                        group="network-admin",
+                        authentication=dict(
+                            algorithm="md5",
+                            password="0x7d425fbf09417c44bca69e1d9e9ce889",
+                            localized_key=True,
+                            priv=dict(privacy_password="0x7d425fbf09417c44bca69e1d9e9ce889"),
+                        ),
+                    ),
+                    dict(
+                        user="snmp_user_1",
+                        group="network-operator",
+                        authentication=dict(
+                            algorithm="md5",
+                            password="0x5632724fb8ac3699296af26281e1d0f1",
+                            localized_key=True,
+                        ),
+                    ),
+                ],
+            ),
         )
         result = self.execute_module(changed=False)
         self.assertEqual(result["parsed"], parsed)
@@ -882,21 +937,13 @@ class TestNxosSnmpServerModule(TestNxosModule):
                     context=dict(name="public", vrf="siteA"),
                     counter=dict(cache=dict(timeout=1800)),
                     drop=dict(unknown_engine_id=True, unknown_user=True),
-                    engine_id=dict(
-                        local="'00:00:00:63:00:01:00:10:20:15:10:03'"
-                    ),
+                    engine_id=dict(local="'00:00:00:63:00:01:00:10:20:15:10:03'"),
                     global_enforce_priv=True,
                     location="lab",
-                    mib=dict(
-                        community_map=dict(
-                            community="public", context="public1"
-                        )
-                    ),
+                    mib=dict(community_map=dict(community="public", context="public1")),
                     packetsize=484,
                     protocol=dict(enable=True),
-                    source_interface=dict(
-                        informs="Ethernet1/1", traps="Ethernet1/2"
-                    ),
+                    source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
                     system_shutdown=True,
                     tcp_session=dict(auth=True),
                 ),
@@ -945,7 +992,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             snmp-server location lab
             snmp-server mib community-map public context public1
             snmp-server source-interface traps Ethernet1/2
-            """
+            """,
         )
         set_module_args(dict(state="gathered"), ignore_provider_arg)
 
@@ -958,9 +1005,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
             engine_id=dict(local="00:00:00:63:00:01:00:10:20:15:10:03"),
             global_enforce_priv=True,
             location="lab",
-            mib=dict(
-                community_map=dict(community="public", context="public1")
-            ),
+            mib=dict(community_map=dict(community="public", context="public1")),
             packetsize=484,
             protocol=dict(enable=True),
             source_interface=dict(informs="Ethernet1/1", traps="Ethernet1/2"),
@@ -973,7 +1018,7 @@ class TestNxosSnmpServerModule(TestNxosModule):
     def test_nxos_snmp_server_gathered_empty(self):
         self.get_config.return_value = dedent(
             """\
-            """
+            """,
         )
         set_module_args(dict(state="gathered"), ignore_provider_arg)
 
