@@ -7,11 +7,14 @@
 # Copyright (c), Ted Timmons <ted@timmons.me>, 2017.
 # Most of this was originally added by other creators in the postgresql_user module.
 #
-# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
+# Simplified BSD License (see simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
 
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
+from datetime import timedelta
+from decimal import Decimal
+
 psycopg2 = None  # This line needs for unit tests
 try:
     import psycopg2
@@ -24,6 +27,8 @@ from ansible.module_utils._text import to_native
 from ansible.module_utils.six import iteritems
 from ansible_collections.community.postgresql.plugins.module_utils.version import LooseVersion
 
+TYPES_NEED_TO_CONVERT = (Decimal, timedelta)
+
 
 def postgres_common_argument_spec():
     """
@@ -39,6 +44,7 @@ def postgres_common_argument_spec():
         port=dict(type='int', default=5432, aliases=['login_port']),
         ssl_mode=dict(default='prefer', choices=['allow', 'disable', 'prefer', 'require', 'verify-ca', 'verify-full']),
         ca_cert=dict(aliases=['ssl_rootcert']),
+        connect_params=dict(default={}, type='dict'),
     )
 
 
@@ -54,7 +60,7 @@ def ensure_required_libs(module):
 def connect_to_db(module, conn_params, autocommit=False, fail_on_conn=True):
     """Connect to a PostgreSQL database.
 
-    Return psycopg2 connection object.
+    Return a tuple containing a psycopg2 connection object and error message / None.
 
     Args:
         module (AnsibleModule) -- object of ansible.module_utils.basic.AnsibleModule class
@@ -64,9 +70,9 @@ def connect_to_db(module, conn_params, autocommit=False, fail_on_conn=True):
         autocommit (bool) -- commit automatically (default False)
         fail_on_conn (bool) -- fail if connection failed or just warn and return None (default True)
     """
-    ensure_required_libs(module)
 
     db_connection = None
+    conn_err = None
     try:
         db_connection = psycopg2.connect(**conn_params)
         if autocommit:
@@ -91,20 +97,19 @@ def connect_to_db(module, conn_params, autocommit=False, fail_on_conn=True):
             module.fail_json(msg='Postgresql server must be at least '
                                  'version 8.4 to support sslrootcert')
 
-        if fail_on_conn:
-            module.fail_json(msg="unable to connect to database: %s" % to_native(e))
-        else:
-            module.warn("PostgreSQL server is unavailable: %s" % to_native(e))
-            db_connection = None
+        conn_err = to_native(e)
 
     except Exception as e:
+        conn_err = to_native(e)
+
+    if conn_err is not None:
         if fail_on_conn:
-            module.fail_json(msg="unable to connect to database: %s" % to_native(e))
+            module.fail_json(msg="unable to connect to database: %s" % conn_err)
         else:
-            module.warn("PostgreSQL server is unavailable: %s" % to_native(e))
+            module.warn("PostgreSQL server is unavailable: %s" % conn_err)
             db_connection = None
 
-    return db_connection
+    return db_connection, conn_err
 
 
 def exec_sql(obj, query, query_params=None, return_bool=False, add_to_executed=True, dont_exec=False):
@@ -172,6 +177,7 @@ def get_conn_params(module, params_dict, warn_db_default=True):
     Kwargs:
         warn_db_default (bool) -- warn that the default DB is used (default True)
     """
+
     # To use defaults values, keyword arguments must be absent, so
     # check which values are empty and don't include in the return dictionary
     params_map = {
@@ -180,20 +186,32 @@ def get_conn_params(module, params_dict, warn_db_default=True):
         "login_password": "password",
         "port": "port",
         "ssl_mode": "sslmode",
-        "ca_cert": "sslrootcert"
+        "ca_cert": "sslrootcert",
     }
 
     # Might be different in the modules:
-    if params_dict.get('db'):
-        params_map['db'] = 'database'
-    elif params_dict.get('database'):
-        params_map['database'] = 'database'
-    elif params_dict.get('login_db'):
-        params_map['login_db'] = 'database'
+    if LooseVersion(psycopg2.__version__) >= LooseVersion('2.7.0'):
+        if params_dict.get('db'):
+            params_map['db'] = 'dbname'
+        elif params_dict.get('database'):
+            params_map['database'] = 'dbname'
+        elif params_dict.get('login_db'):
+            params_map['login_db'] = 'dbname'
+        else:
+            if warn_db_default:
+                module.warn('Database name has not been passed, '
+                            'used default database to connect to.')
     else:
-        if warn_db_default:
-            module.warn('Database name has not been passed, '
-                        'used default database to connect to.')
+        if params_dict.get('db'):
+            params_map['db'] = 'database'
+        elif params_dict.get('database'):
+            params_map['database'] = 'database'
+        elif params_dict.get('login_db'):
+            params_map['login_db'] = 'database'
+        else:
+            if warn_db_default:
+                module.warn('Database name has not been passed, '
+                            'used default database to connect to.')
 
     kw = dict((params_map[k], v) for (k, v) in iteritems(params_dict)
               if k in params_map and v != '' and v is not None)
@@ -206,9 +224,36 @@ def get_conn_params(module, params_dict, warn_db_default=True):
     if is_localhost and params_dict["login_unix_socket"] != "":
         kw["host"] = params_dict["login_unix_socket"]
 
+    # If connect_params is specified, merge it together
+    if params_dict.get("connect_params"):
+        kw.update(params_dict["connect_params"])
+
     return kw
 
 
+class PgRole():
+    def __init__(self, module, cursor, name):
+        self.module = module
+        self.cursor = cursor
+        self.name = name
+        self.memberof = self.__fetch_members()
+
+    def __fetch_members(self):
+        query = ("SELECT ARRAY(SELECT b.rolname FROM "
+                 "pg_catalog.pg_auth_members m "
+                 "JOIN pg_catalog.pg_roles b ON (m.roleid = b.oid) "
+                 "WHERE m.member = r.oid) "
+                 "FROM pg_catalog.pg_roles r "
+                 "WHERE r.rolname = %(dst_role)s")
+
+        res = exec_sql(self, query, query_params={'dst_role': self.name},
+                       add_to_executed=False)
+        if res:
+            return res[0][0]
+        else:
+            return []
+
+
 class PgMembership(object):
     def __init__(self, module, cursor, groups, target_roles, fail_on_role=True):
         self.module = module
@@ -228,8 +273,9 @@ class PgMembership(object):
             self.granted[group] = []
 
             for role in self.target_roles:
+                role_obj = PgRole(self.module, self.cursor, role)
                 # If role is in a group now, pass:
-                if self.__check_membership(group, role):
+                if group in role_obj.memberof:
                     continue
 
                 query = 'GRANT "%s" TO "%s"' % (group, role)
@@ -245,8 +291,9 @@ class PgMembership(object):
             self.revoked[group] = []
 
             for role in self.target_roles:
+                role_obj = PgRole(self.module, self.cursor, role)
                 # If role is not in a group now, pass:
-                if not self.__check_membership(group, role):
+                if group not in role_obj.memberof:
                     continue
 
                 query = 'REVOKE "%s" FROM "%s"' % (group, role)
@@ -257,39 +304,48 @@ class PgMembership(object):
 
         return self.changed
 
-    def __check_membership(self, src_role, dst_role):
-        query = ("SELECT ARRAY(SELECT b.rolname FROM "
-                 "pg_catalog.pg_auth_members m "
-                 "JOIN pg_catalog.pg_roles b ON (m.roleid = b.oid) "
-                 "WHERE m.member = r.oid) "
-                 "FROM pg_catalog.pg_roles r "
-                 "WHERE r.rolname = %(dst_role)s")
+    def match(self):
+        for role in self.target_roles:
+            role_obj = PgRole(self.module, self.cursor, role)
 
-        res = exec_sql(self, query, query_params={'dst_role': dst_role}, add_to_executed=False)
-        membership = []
-        if res:
-            membership = res[0][0]
+            desired_groups = set(self.groups)
+            current_groups = set(role_obj.memberof)
+            # 1. Get groups that the role is member of but not in self.groups and revoke them
+            groups_to_revoke = current_groups - desired_groups
+            for group in groups_to_revoke:
+                query = 'REVOKE "%s" FROM "%s"' % (group, role)
+                self.changed = exec_sql(self, query, return_bool=True)
+                if group in self.revoked:
+                    self.revoked[group].append(role)
+                else:
+                    self.revoked[group] = [role]
 
-        if not membership:
-            return False
+            # 2. Filter out groups that in self.groups and
+            # the role is already member of and grant the rest
+            groups_to_grant = desired_groups - current_groups
+            for group in groups_to_grant:
+                query = 'GRANT "%s" TO "%s"' % (group, role)
+                self.changed = exec_sql(self, query, return_bool=True)
+                if group in self.granted:
+                    self.granted[group].append(role)
+                else:
+                    self.granted[group] = [role]
 
-        if src_role in membership:
-            return True
-
-        return False
+        return self.changed
 
     def __check_roles_exist(self):
-        existent_groups = self.__roles_exist(self.groups)
+        if self.groups:
+            existent_groups = self.__roles_exist(self.groups)
+
+            for group in self.groups:
+                if group not in existent_groups:
+                    if self.fail_on_role:
+                        self.module.fail_json(msg="Role %s does not exist" % group)
+                    else:
+                        self.module.warn("Role %s does not exist, pass" % group)
+                        self.non_existent_roles.append(group)
+
         existent_roles = self.__roles_exist(self.target_roles)
-
-        for group in self.groups:
-            if group not in existent_groups:
-                if self.fail_on_role:
-                    self.module.fail_json(msg="Role %s does not exist" % group)
-                else:
-                    self.module.warn("Role %s does not exist, pass" % group)
-                    self.non_existent_roles.append(group)
-
         for role in self.target_roles:
             if role not in existent_roles:
                 if self.fail_on_role:
@@ -307,7 +363,8 @@ class PgMembership(object):
                         self.module.warn("Role role '%s' is a member of role '%s', pass" % (role, role))
 
         # Update role lists, excluding non existent roles:
-        self.groups = [g for g in self.groups if g not in self.non_existent_roles]
+        if self.groups:
+            self.groups = [g for g in self.groups if g not in self.non_existent_roles]
 
         self.target_roles = [r for r in self.target_roles if r not in self.non_existent_roles]
 
@@ -315,3 +372,80 @@ class PgMembership(object):
         tmp = ["'" + x + "'" for x in roles]
         query = "SELECT rolname FROM pg_roles WHERE rolname IN (%s)" % ','.join(tmp)
         return [x[0] for x in exec_sql(self, query, add_to_executed=False)]
+
+
+def set_search_path(cursor, search_path):
+    """Set session's search_path.
+
+    Args:
+        cursor (Psycopg2 cursor): Database cursor object.
+        search_path (str): String containing comma-separated schema names.
+    """
+    cursor.execute('SET search_path TO %s' % search_path)
+
+
+def convert_elements_to_pg_arrays(obj):
+    """Convert list elements of the passed object
+    to PostgreSQL arrays represented as strings.
+
+    Args:
+        obj (dict or list): Object whose elements need to be converted.
+
+    Returns:
+        obj (dict or list): Object with converted elements.
+    """
+    if isinstance(obj, dict):
+        for (key, elem) in iteritems(obj):
+            if isinstance(elem, list):
+                obj[key] = list_to_pg_array(elem)
+
+    elif isinstance(obj, list):
+        for i, elem in enumerate(obj):
+            if isinstance(elem, list):
+                obj[i] = list_to_pg_array(elem)
+
+    return obj
+
+
+def list_to_pg_array(elem):
+    """Convert the passed list to PostgreSQL array
+    represented as a string.
+
+    Args:
+        elem (list): List that needs to be converted.
+
+    Returns:
+        elem (str): String representation of PostgreSQL array.
+    """
+    elem = str(elem).strip('[]')
+    elem = '{' + elem + '}'
+    return elem
+
+
+def convert_to_supported(val):
+    """Convert unsupported type to appropriate.
+    Args:
+        val (any) -- Any value fetched from database.
+    Returns value of appropriate type.
+    """
+    if isinstance(val, Decimal):
+        return float(val)
+
+    elif isinstance(val, timedelta):
+        return str(val)
+
+    return val  # By default returns the same value
+
+
+def get_server_version(conn):
+    """Get server version.
+
+    Args:
+        conn (psycopg.Connection) -- Psycopg connection object.
+
+    Returns server version (int).
+    """
+    if LooseVersion(psycopg2.__version__) >= LooseVersion('3.0.0'):
+        return conn.info.server_version
+    else:
+        return conn.server_version
