@@ -16,7 +16,7 @@ description:
 author:
     - Ruben Tsirunyan (@rubentsirunyan)
 requirements:
-    - "zabbix-api >= 0.5.4"
+    - "python >= 2.6"
 
 options:
     name:
@@ -77,12 +77,12 @@ options:
             - Works only with Zabbix versions 3.4 or newer.
         default: 3
     attempt_interval:
-        type: 'int'
+        type: 'str'
         description:
             - The interval between retry attempts.
-            - Possible range is 0-60.
+            - Possible range is 0-60s in Zabbix < 5.0 or 0-1h in Zabbix >= 5.0.
             - Works only with Zabbix versions 3.4 or newer.
-        default: 10
+        default: 10s
     script_name:
         type: 'str'
         description:
@@ -277,12 +277,31 @@ extends_documentation_fragment:
 RETURN = r''' # '''
 
 EXAMPLES = r'''
+# Set following variables for Zabbix Server host in play or inventory
+- name: Set connection specific variables
+  set_fact:
+    ansible_network_os: community.zabbix.zabbix
+    ansible_connection: httpapi
+    ansible_httpapi_port: 80
+    ansible_httpapi_use_ssl: false
+    ansible_httpapi_validate_certs: false
+    ansible_zabbix_url_path: 'zabbixeu'  # If Zabbix WebUI runs on non-default (zabbix) path ,e.g. http://<FQDN>/zabbixeu
+
+# If you want to use Username and Password to be authenticated by Zabbix Server
+- name: Set credentials to access Zabbix Server API
+  set_fact:
+    ansible_user: Admin
+    ansible_httpapi_pass: zabbix
+
+# If you want to use API token to be authenticated by Zabbix Server
+# https://www.zabbix.com/documentation/current/en/manual/web_interface/frontend_sections/administration/general#api-tokens
+- name: Set API token
+  set_fact:
+    ansible_zabbix_auth_key: 8ec0d52432c15c91fcafe9888500cf9a607f44091ab554dbee860f6b44fac895
+
 - name: 'Create an email mediatype with SMTP authentication'
   community.zabbix.zabbix_mediatype:
     name: "Ops email"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'email'
     smtp_server: 'example.com'
     smtp_server_port: 2000
@@ -294,9 +313,6 @@ EXAMPLES = r'''
 - name: 'Create a script mediatype'
   community.zabbix.zabbix_mediatype:
     name: "my script"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'script'
     script_name: 'my_script.py'
     script_params:
@@ -306,9 +322,6 @@ EXAMPLES = r'''
 - name: 'Create a jabber mediatype'
   community.zabbix.zabbix_mediatype:
     name: "My jabber"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'jabber'
     username: 'jabber_id'
     password: 'jabber_pass'
@@ -316,9 +329,6 @@ EXAMPLES = r'''
 - name: 'Create a SMS mediatype'
   community.zabbix.zabbix_mediatype:
     name: "My SMS Mediatype"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'sms'
     gsm_modem: '/dev/ttyS0'
 
@@ -326,9 +336,6 @@ EXAMPLES = r'''
 - name: 'Create a webhook mediatype'
   community.zabbix.zabbix_mediatype:
     name: "My webhook Mediatype"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'webhook'
     webhook_script: "{{ lookup('file', 'slack.js') }}"
     webhook_params:
@@ -345,9 +352,6 @@ EXAMPLES = r'''
 - name: 'Create an email mediatype with message templates'
   community.zabbix.zabbix_mediatype:
     name: "Ops email"
-    server_url: "http://example.com/zabbix/"
-    login_user: Admin
-    login_password: "zabbix"
     type: 'email'
     smtp_email: 'ops@example.com'
     message_templates:
@@ -641,7 +645,7 @@ def main():
         status=dict(type='str', default='enabled', choices=['enabled', 'disabled'], required=False),
         max_sessions=dict(type='int', default=1, required=False),
         max_attempts=dict(type='int', default=3, required=False),
-        attempt_interval=dict(type='int', default=10, required=False),
+        attempt_interval=dict(type='str', default='10s', required=False),
         # Script
         script_name=dict(type='str', required=False),
         script_params=dict(type='list', required=False),
@@ -712,6 +716,12 @@ def main():
         supports_check_mode=True
     )
 
+    zabbix_utils.require_creds_params(module)
+
+    for p in ['server_url', 'login_user', 'login_password', 'timeout', 'validate_certs']:
+        if p in module.params:
+            module.warn('Option "%s" is deprecated with the move to httpapi connection and will be removed in the next release' % p)
+
     state = module.params['state']
     name = module.params['name']
 
