@@ -24,12 +24,12 @@ ANSIBLE_METADATA = {'status': ['preview'],
 DOCUMENTATION = '''
 ---
 module: fmgr_pkg_header_shapingpolicy
-short_description: Configure shaping policies.
+short_description: no description
 description:
     - This module is able to configure a FortiManager device.
     - Examples include all parameters and values which need to be adjusted to data sources before usage.
 
-version_added: "2.10"
+version_added: "1.0.0"
 author:
     - Link Zheng (@chillancezen)
     - Jie Xue (@JieX19)
@@ -58,12 +58,15 @@ options:
           - set
           - add
     bypass_validation:
-        description: only set to True when module schema diffs with FortiManager API structure, module continues to execute without validating parameters
+        description: |
+          only set to True when module schema diffs with FortiManager API structure,
+           module continues to execute without validating parameters
         required: false
         type: bool
         default: false
     workspace_locking_adom:
-        description: the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
+        description: |
+          the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
         required: false
         type: str
     workspace_locking_timeout:
@@ -102,7 +105,7 @@ options:
                 type: str
                 description: no description
             application:
-                description: no description
+                description: description
                 type: int
             class-id:
                 type: int
@@ -242,6 +245,15 @@ options:
             internet-service-src-name:
                 type: str
                 description: no description
+            class-id-reverse:
+                type: int
+                description: no description
+            service-type:
+                type: str
+                description: no description
+                choices:
+                    - 'service'
+                    - 'internet-service'
 
 '''
 
@@ -255,7 +267,7 @@ EXAMPLES = '''
       ansible_httpapi_validate_certs: False
       ansible_httpapi_port: 443
    tasks:
-    - name: Configure shaping policies.
+    - name: no description
       fmgr_pkg_header_shapingpolicy:
          bypass_validation: False
          workspace_locking_adom: <value in [global, custom adom including root]>
@@ -307,6 +319,8 @@ EXAMPLES = '''
             uuid: <value of string>
             internet-service-name: <value of string>
             internet-service-src-name: <value of string>
+            class-id-reverse: <value of integer>
+            service-type: <value in [service, internet-service]>
 
 '''
 
@@ -352,6 +366,11 @@ def main():
             'required': False,
             'default': False
         },
+        'forticloud_access_token': {
+            'type': 'str',
+            'required': False,
+            'no_log': True
+        },
         'proposed_method': {
             'type': 'str',
             'required': False,
@@ -406,7 +425,8 @@ def main():
                 '6.4.0': True,
                 '6.4.2': True,
                 '6.4.5': True,
-                '7.0.0': True
+                '7.0.0': True,
+                '7.2.0': True
             },
             'options': {
                 'app-category': {
@@ -419,7 +439,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -433,7 +454,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -447,7 +469,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'int'
                 },
@@ -461,7 +484,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'int'
                 },
@@ -475,7 +499,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -489,7 +514,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -507,7 +533,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -525,7 +552,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -539,7 +567,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -553,7 +582,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -567,7 +597,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -581,7 +612,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -595,7 +627,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -609,7 +642,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'int'
                 },
@@ -623,7 +657,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -641,7 +676,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -655,7 +691,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -669,7 +706,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -683,7 +721,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -697,7 +736,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -715,7 +755,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -729,7 +770,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -743,7 +785,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -757,7 +800,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -771,7 +815,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         '4',
@@ -789,7 +834,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -803,7 +849,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -817,7 +864,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -831,7 +879,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -845,7 +894,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -859,7 +909,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -873,7 +924,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -891,7 +943,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -905,7 +958,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -919,7 +973,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -937,7 +992,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -951,7 +1007,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -965,7 +1022,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -979,7 +1037,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -992,7 +1051,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -1002,7 +1062,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
                 },
@@ -1012,9 +1073,28 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'type': 'str'
+                },
+                'class-id-reverse': {
+                    'required': False,
+                    'revision': {
+                        '7.2.0': True
+                    },
+                    'type': 'int'
+                },
+                'service-type': {
+                    'required': False,
+                    'revision': {
+                        '7.2.0': True
+                    },
+                    'choices': [
+                        'service',
+                        'internet-service'
+                    ],
+                    'type': 'str'
                 }
             }
 
@@ -1041,6 +1121,8 @@ def main():
     if module._socket_path:
         connection = Connection(module._socket_path)
         connection.set_option('enable_log', module.params['enable_log'] if 'enable_log' in module.params else False)
+        connection.set_option('forticloud_access_token',
+                              module.params['forticloud_access_token'] if 'forticloud_access_token' in module.params else None)
         fmgr = NAPIManager(jrpc_urls, perobject_jrpc_urls, module_primary_key, url_params, module, connection, top_level_schema_name='data')
         fmgr.validate_parameters(params_validation_blob)
         fmgr.process_curd(argument_specs=module_arg_spec)
