@@ -24,12 +24,12 @@ ANSIBLE_METADATA = {'status': ['preview'],
 DOCUMENTATION = '''
 ---
 module: fmgr_system_locallog_syslogd2_filter
-short_description: Filter for syslog logging.
+short_description: no description
 description:
     - This module is able to configure a FortiManager device.
     - Examples include all parameters and values which need to be adjusted to data sources before usage.
 
-version_added: "2.10"
+version_added: "1.0.0"
 author:
     - Link Zheng (@chillancezen)
     - Jie Xue (@JieX19)
@@ -58,12 +58,15 @@ options:
           - set
           - add
     bypass_validation:
-        description: only set to True when module schema diffs with FortiManager API structure, module continues to execute without validating parameters
+        description: |
+          only set to True when module schema diffs with FortiManager API structure,
+           module continues to execute without validating parameters
         required: false
         type: bool
         default: false
     workspace_locking_adom:
-        description: the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
+        description: |
+          the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
         required: false
         type: str
     workspace_locking_timeout:
@@ -95,9 +98,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log device configuration message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -105,9 +108,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Managered devices operations messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -115,9 +118,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer disk quota messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -125,9 +128,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log deployment manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -135,9 +138,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log device manager messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -145,9 +148,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer ediscovery messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -155,9 +158,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log endpoint manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -165,9 +168,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log event messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -175,9 +178,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer event handler messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -185,9 +188,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -195,9 +198,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer HA messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -205,9 +208,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer system messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -215,9 +218,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log FortiGuard service message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -225,9 +228,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log FGFM protocol message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -235,9 +238,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Whether to log fips messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -245,9 +248,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log web service messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -255,9 +258,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log FortiMail manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -265,9 +268,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log firmware manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -275,9 +278,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer FortiView messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -285,9 +288,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log global database message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -295,9 +298,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log HA message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -305,9 +308,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer hcache messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -315,9 +318,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log debug IO log message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -325,9 +328,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log the status of log daemon.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -335,9 +338,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer log DB messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -345,18 +348,18 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer log device messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
             logfile:
                 type: str
                 description:
-                 - 'Log Fortianalyzer log file messages.'
-                 - 'enable - Enable setting.'
-                 - 'disable - Disable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'enable'
                     - 'disable'
@@ -364,9 +367,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer logging messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -374,9 +377,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log log and report manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -384,9 +387,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log object configuration change message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -394,9 +397,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer report messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -404,9 +407,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log revision history message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -414,9 +417,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log real-time monitor message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -424,9 +427,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log firewall objects message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -434,9 +437,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log policy console message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -444,9 +447,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log script manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -454,9 +457,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log VPN console message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -464,9 +467,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log system manager message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -474,9 +477,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log web portal message.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -484,9 +487,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log Fortianalyzer incident messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -494,9 +497,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Log aid messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -504,9 +507,9 @@ options:
                 type: str
                 default: 'enable'
                 description:
-                 - 'Docker application generic messages.'
-                 - 'disable - Disable setting.'
-                 - 'enable - Enable setting.'
+                 - no description
+                 - no description
+                 - no description
                 choices:
                     - 'disable'
                     - 'enable'
@@ -523,7 +526,7 @@ EXAMPLES = '''
       ansible_httpapi_validate_certs: False
       ansible_httpapi_port: 443
    tasks:
-    - name: Filter for syslog logging.
+    - name: no description
       fmgr_system_locallog_syslogd2_filter:
          bypass_validation: False
          workspace_locking_adom: <value in [global, custom adom including root]>
@@ -665,7 +668,8 @@ def main():
                 '6.4.0': True,
                 '6.4.2': True,
                 '6.4.5': True,
-                '7.0.0': True
+                '7.0.0': True,
+                '7.2.0': True
             },
             'options': {
                 'devcfg': {
@@ -678,7 +682,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -696,7 +701,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -714,7 +720,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -732,7 +739,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -750,7 +758,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -768,7 +777,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -786,7 +796,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -804,7 +815,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -822,7 +834,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -840,7 +853,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -858,7 +872,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -876,7 +891,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -894,7 +910,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -912,7 +929,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -930,7 +948,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -948,7 +967,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -966,7 +986,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -984,7 +1005,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1002,7 +1024,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1020,7 +1043,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1038,7 +1062,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1056,7 +1081,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1074,7 +1100,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1092,7 +1119,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1110,7 +1138,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1128,7 +1157,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1146,7 +1176,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'enable',
@@ -1164,7 +1195,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1182,7 +1214,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1200,7 +1233,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1218,7 +1252,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1236,7 +1271,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1254,7 +1290,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1272,7 +1309,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1290,7 +1328,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1308,7 +1347,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1326,7 +1366,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1344,7 +1385,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1362,7 +1404,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1376,7 +1419,8 @@ def main():
                         '6.4.0': True,
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1389,7 +1433,8 @@ def main():
                     'revision': {
                         '6.4.2': True,
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
@@ -1401,7 +1446,8 @@ def main():
                     'required': False,
                     'revision': {
                         '6.4.5': True,
-                        '7.0.0': True
+                        '7.0.0': True,
+                        '7.2.0': True
                     },
                     'choices': [
                         'disable',
