@@ -1,44 +1,23 @@
-# This code is part of Ansible, but is an independent component.
-# This particular file snippet, and this file snippet only, is BSD licensed.
-# Modules you write using this snippet, which is embedded dynamically by Ansible
-# still belong to the author of the module, and may assign their own license
-# to the complete work.
-#
-# (c) 2019 Fortinet, Inc
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without modification,
-# are permitted provided that the following conditions are met:
-#
-#    * Redistributions of source code must retain the above copyright
-#      notice, this list of conditions and the following disclaimer.
-#    * Redistributions in binary form must reproduce the above copyright notice,
-#      this list of conditions and the following disclaimer in the documentation
-#      and/or other materials provided with the distribution.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
-# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Copyright (c) 2022 Fortinet
+# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
 DOCUMENTATION = """
 ---
-author:
-    - Miguel Angel Munoz (@magonzalez)
-httpapi : fortios
+name: fortios
 short_description: HttpApi Plugin for Fortinet FortiOS Appliance or VM
 description:
   - This HttpApi plugin provides methods to connect to Fortinet FortiOS Appliance or VM via REST API
-version_added: "2.9"
+author:
+  - Miguel Angel Munoz (@magonzalez)
+version_added: "2.0.0"
 """
 
 import json
@@ -58,9 +37,13 @@ class HttpApi(HttpApiBase):
         self._ccsrftoken = ''
         self._system_version = None
         self._ansible_fos_version = 'v6.0.0'
-        self._ansible_galaxy_version = '2.1.3'
+        self._ansible_galaxy_version = '2.2.1'
         self._log = None
 
+        # workaround for HttApiBase: it always tries to do a basic auth for the first request to update auth headers
+        # for token based auth, we need the Accept header to request the right data format
+        self._conn._auth = {"Accept": "application/json"}
+
     def log(self, msg):
         log_enabled = self._conn.get_option('enable_log')
         if not log_enabled:
@@ -100,19 +83,17 @@ class HttpApi(HttpApiBase):
                 raise Exception('Wrong credentials. Please check')
         # If we succeed to login, we retrieve the system status first
         else:
-            self.log('login with access token')
-            self.send_request(url='/logincheck')
-            status, dummy = self.send_request(url='/api/v2/cmdb/system/interface?vdom=root&action=schema')
-
+            self.log('Calling Fortios with access tokens, no login needed')
+            url = '/api/v2/cmdb/system/interface?vdom=root&action=schema'
+            status, response = self.send_request(url=url)
             if status == 401:
                 raise Exception('Invalid access token. Please check')
 
-        self.update_system_version()
-
     def logout(self):
         """ Call to implement session logout."""
         self.log('logout')
-        self.send_request(url='/logout', method="POST")
+        if not self.get_access_token():
+            self.send_request(url='/logout', method="POST")
 
     def update_auth(self, response, response_text):
         """
@@ -121,7 +102,7 @@ class HttpApi(HttpApiBase):
         :param response_text Unused_input.
         :return: Dictionary containing headers
         """
-
+        self.log('update_auth')
         if self.get_access_token() is None:
             headers = {}
 
@@ -185,17 +166,18 @@ class HttpApi(HttpApiBase):
         data = message_kwargs.get('data', '')
         method = message_kwargs.get('method', 'GET')
         params = message_kwargs.get('params', {})
+        headers = dict(message_kwargs.get('headers', {}))
+        headers.update({'Accept': 'application/json'})
 
         url = self._concat_params(url, params)
-        self.log('send request: METHOD:%s URL:%s DATA:%s' % (method, url, data))
-
+        self.log('send request: METHOD:[%s] URL:[%s] DATA:[%s], headers:[%s]' % (method, url, data, headers))
 
         try:
-            response, response_data = self.connection.send(url, data, method=method)
+            response, response_data = self.connection.send(url, data, method=method, headers=headers)
 
             json_formatted = to_text(response_data.getvalue())
 
-            self.log("response data: %s" % (json_formatted))
+            self.log("response data: %s..." % (json_formatted[:200]))
             return response.status, json_formatted
         except Exception as err:
             raise Exception(err)
@@ -204,6 +186,7 @@ class HttpApi(HttpApiBase):
         """
         retrieve the system status of fortigate device
         """
+        self.log('checking system_version')
         check_system_status = self._conn.get_option('check_system_status') if 'check_system_status' in self._conn._options else True
         if not check_system_status or self._system_version:
             return
