@@ -71,6 +71,7 @@ options:
   client_id:
     description:
     - The unique client ID of the Connector.
+    - The connector ID.
     type: str
 
   subscription_id:
@@ -159,6 +160,13 @@ options:
     - The password for the Connector.
     type: str
 
+  storage_account:
+    description:
+    - The storage account can be created automatically.
+    - When C(storage_account) is not set, the name is constructed by appending 'sa' to the connector C(name).
+    - Storage account name must be between 3 and 24 characters in length and use numbers and lower-case letters only.
+    type: str
+    version_added: '21.17.0'
 '''
 
 EXAMPLES = """
@@ -227,7 +235,6 @@ try:
     from azure.mgmt.network import NetworkManagementClient
     from azure.mgmt.storage import StorageManagementClient
     from azure.mgmt.resource.resources.models import Deployment
-    from azure.mgmt.resource.resources.models import DeploymentProperties
     from azure.common.client_factory import get_client_from_cli_profile
     from msrestazure.azure_exceptions import CloudError
     HAS_AZURE_LIB = True
@@ -264,6 +271,7 @@ class NetAppCloudManagerConnectorAzure(object):
             proxy_password=dict(required=False, type='str', default='', no_log=True),
             admin_username=dict(required=True, type='str'),
             admin_password=dict(required=True, type='str', no_log=True),
+            storage_account=dict(required=False, type='str'),
         ))
 
         self.module = AnsibleModule(
@@ -282,7 +290,8 @@ class NetAppCloudManagerConnectorAzure(object):
 
         self.na_helper = NetAppModule()
         self.parameters = self.na_helper.set_parameters(self.module.params)
-
+        if 'storage_account' not in self.parameters or self.parameters['storage_account'] == "":
+            self.parameters['storage_account'] = self.parameters['name'].lower() + 'sa'
         self.rest_api = CloudManagerRestAPI(self.module)
 
     def get_deploy_azure_vm(self):
@@ -296,7 +305,6 @@ class NetAppCloudManagerConnectorAzure(object):
         exists = False
 
         resource_client = get_client_from_cli_profile(ResourceManagementClient)
-
         try:
             exists = resource_client.deployments.check_existence(self.parameters['resource_group'], self.parameters['name'])
 
@@ -315,25 +323,30 @@ class NetAppCloudManagerConnectorAzure(object):
         """
 
         user_data, client_id = self.register_agent_to_service()
-
         template = json.loads(self.na_helper.call_template())
         params = json.loads(self.na_helper.call_parameters())
-
         params['adminUsername']['value'] = self.parameters['admin_username']
         params['adminPassword']['value'] = self.parameters['admin_password']
         params['customData']['value'] = json.dumps(user_data)
         params['location']['value'] = self.parameters['location']
         params['virtualMachineName']['value'] = self.parameters['name']
+        params['storageAccount']['value'] = self.parameters['storage_account']
         if self.rest_api.environment == 'stage':
             params['environment']['value'] = self.rest_api.environment
-        if self.parameters.get('vnet_resource_group') is not None:
-            network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_name'])
+        if '/subscriptions' in self.parameters['vnet_name']:
+            network = self.parameters['vnet_name']
         else:
-            network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_name'])
+            if self.parameters.get('vnet_resource_group') is not None:
+                network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
+                    self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_name'])
+            else:
+                network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
+                    self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_name'])
 
-        subnet = '%s/subnets/%s' % (network, self.parameters['subnet_name'])
+        if '/subscriptions' in self.parameters['subnet_name']:
+            subnet = self.parameters['subnet_name']
+        else:
+            subnet = '%s/subnets/%s' % (network, self.parameters['subnet_name'])
 
         if self.parameters.get('network_security_resource_group') is not None:
             network_security_group_name = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/networkSecurityGroups/%s' % (
@@ -384,7 +397,15 @@ class NetAppCloudManagerConnectorAzure(object):
         if retries == 0:
             # Taking too long for status to be active
             return self.module.fail_json(msg="Taking too long for OCCM agent to be active or not properly setup")
-        return client_id
+
+        try:
+            compute_client = get_client_from_cli_profile(ComputeManagementClient)
+            vm = compute_client.virtual_machines.get(self.parameters['resource_group'], self.parameters['name'])
+        except CloudError as error:
+            return self.module.fail_json(msg="Error in deploy_azure (get identity): %s" % to_native(error), exception=traceback.format_exc())
+
+        principal_id = vm.identity.principal_id
+        return client_id, principal_id
 
     def register_agent_to_service(self):
         """
@@ -392,14 +413,20 @@ class NetAppCloudManagerConnectorAzure(object):
         :return: UserData, ClientID
         """
 
-        if self.parameters.get('vnet_resource_group') is not None:
-            network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_name'])
+        if '/subscriptions' in self.parameters['vnet_name']:
+            network = self.parameters['vnet_name']
         else:
-            network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
-                self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_name'])
+            if self.parameters.get('vnet_resource_group') is not None:
+                network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
+                    self.parameters['subscription_id'], self.parameters['vnet_resource_group'], self.parameters['vnet_name'])
+            else:
+                network = '/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/virtualNetworks/%s' % (
+                    self.parameters['subscription_id'], self.parameters['resource_group'], self.parameters['vnet_name'])
 
-        subnet = '%s/subnets/%s' % (network, self.parameters['subnet_name'])
+        if '/subscriptions' in self.parameters['subnet_name']:
+            subnet = self.parameters['subnet_name']
+        else:
+            subnet = '%s/subnets/%s' % (network, self.parameters['subnet_name'])
 
         if self.parameters.get('account_id') is None:
             response, error = self.na_helper.get_or_create_account(self.rest_api)
@@ -485,7 +512,7 @@ class NetAppCloudManagerConnectorAzure(object):
             storage_client = get_client_from_cli_profile(StorageManagementClient)
             storage_client.storage_accounts.delete(
                 self.parameters['resource_group'],
-                self.parameters['name'] + 'sa')
+                self.parameters['storage_account'])
         except CloudError as error:
             self.module.fail_json(msg=to_native(error), exception=traceback.format_exc())
 
@@ -526,17 +553,10 @@ class NetAppCloudManagerConnectorAzure(object):
         if retries == 0:
             # Taking too long for terminating OCCM
             return self.module.fail_json(msg="Taking too long for instance to finish terminating")
-
-        client = self.rest_api.format_cliend_id(self.parameters['client_id'])
-        delete_occum_url = "%s/agents-mgmt/agent/%s" % client
-        headers = {
-            "X-User-Token": self.rest_api.token_type + " " + self.rest_api.token,
-            "X-Tenancy-Account-Id": self.parameters['account_id']
-        }
-
-        response, error, dummy = self.rest_api.delete(delete_occum_url, None, header=headers)
-        if error is not None:
-            self.module.fail_json(msg="Error: unexpected response on deleting OCCM: %s, %s" % (str(error), str(response)))
+        client = self.rest_api.format_client_id(self.parameters['client_id'])
+        error = self.na_helper.delete_occm_agents(self.rest_api, [{'agentId': client}])
+        if error:
+            self.module.fail_json(msg="Error: unexpected response on deleting OCCM: %s" % (str(error)))
 
     def apply(self):
         """
@@ -544,9 +564,10 @@ class NetAppCloudManagerConnectorAzure(object):
         :return: None
         """
         client_id = None
+        principal_id = None
         if not self.module.check_mode:
             if self.parameters['state'] == 'present':
-                client_id = self.deploy_azure()
+                client_id, principal_id = self.deploy_azure()
                 self.na_helper.changed = True
             elif self.parameters['state'] == 'absent':
                 get_deploy = self.get_deploy_azure_vm()
@@ -554,7 +575,7 @@ class NetAppCloudManagerConnectorAzure(object):
                     self.delete_azure_occm()
                     self.na_helper.changed = True
 
-        self.module.exit_json(changed=self.na_helper.changed, msg={'client_id': client_id})
+        self.module.exit_json(changed=self.na_helper.changed, msg={'client_id': client_id, 'principal_id': principal_id})
 
 
 def main():
