@@ -3,7 +3,7 @@
 create Autosupport module to enable, disable or modify
 """
 
-# (c) 2018-2021, NetApp, Inc
+# (c) 2018-2022, NetApp, Inc
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
@@ -20,116 +20,116 @@ author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 options:
   state:
     description:
-    - Specifies whether the AutoSupport daemon is present or absent.
-    - When this setting is absent, delivery of all AutoSupport messages is turned off.
+      - Specifies whether the AutoSupport daemon is present or absent.
+      - When this setting is absent, delivery of all AutoSupport messages is turned off.
     choices: ['present', 'absent']
     type: str
     default: present
   node_name:
     description:
-    - The name of the filer that owns the AutoSupport Configuration.
+      - The name of the filer that owns the AutoSupport Configuration.
     required: true
     type: str
   transport:
     description:
-    - The name of the transport protocol used to deliver AutoSupport messages
+      - The name of the transport protocol used to deliver AutoSupport messages.
     choices: ['http', 'https', 'smtp']
     type: str
   noteto:
     description:
-    - Specifies up to five recipients of short AutoSupport e-mail messages.
+      - Specifies up to five recipients of short AutoSupport e-mail messages.
     type: list
     elements: str
   post_url:
     description:
-    - The URL used to deliver AutoSupport messages via HTTP POST
+      - The URL used to deliver AutoSupport messages via HTTP POST.
     type: str
   mail_hosts:
     description:
-    - List of mail server(s) used to deliver AutoSupport messages via SMTP.
-    - Both host names and IP addresses may be used as valid input.
+      - List of mail server(s) used to deliver AutoSupport messages via SMTP.
+      - Both host names and IP addresses may be used as valid input.
     type: list
     elements: str
   support:
     description:
-    - Specifies whether AutoSupport notification to technical support is enabled.
+      - Specifies whether AutoSupport notification to technical support is enabled.
     type: bool
   from_address:
     description:
-    - specify the e-mail address from which the node sends AutoSupport messages
+      - specify the e-mail address from which the node sends AutoSupport messages.
     version_added: 2.8.0
     type: str
   partner_addresses:
     description:
-    - Specifies up to five partner vendor recipients of full AutoSupport e-mail messages.
+      - Specifies up to five partner vendor recipients of full AutoSupport e-mail messages.
     version_added: 2.8.0
     type: list
     elements: str
   to_addresses:
     description:
-    - Specifies up to five recipients of full AutoSupport e-mail messages.
+      - Specifies up to five recipients of full AutoSupport e-mail messages.
     version_added: 2.8.0
     type: list
     elements: str
   proxy_url:
     description:
-    - specify an HTTP or HTTPS proxy if the 'transport' parameter is set to HTTP or HTTPS and your organization uses a proxy.
-    - If authentication is required, use the format "username:password@host:port".
+      - specify an HTTP or HTTPS proxy if the 'transport' parameter is set to HTTP or HTTPS and your organization uses a proxy.
+      - If authentication is required, use the format "username:password@host:port".
     version_added: 2.8.0
     type: str
   hostname_in_subject:
     description:
-    - Specify whether the hostname of the node is included in the subject line of the AutoSupport message.
+      - Specify whether the hostname of the node is included in the subject line of the AutoSupport message.
     type: bool
     version_added: 2.8.0
   nht_data_enabled:
     description:
-    - Specify whether the disk health data is collected as part of the AutoSupport data.
+      - Specify whether the disk health data is collected as part of the AutoSupport data.
     type: bool
     version_added: '21.5.0'
   perf_data_enabled:
     description:
-    - Specify whether the performance data is collected as part of the AutoSupport data.
+      - Specify whether the performance data is collected as part of the AutoSupport data.
     type: bool
     version_added: '21.5.0'
   retry_count:
     description:
-    - Specify the maximum number of delivery attempts for an AutoSupport message.
+      - Specify the maximum number of delivery attempts for an AutoSupport message.
     type: int
     version_added: '21.5.0'
   reminder_enabled:
     description:
-    - Specify whether AutoSupport reminders are enabled or disabled.
+      - Specify whether AutoSupport reminders are enabled or disabled.
     type: bool
     version_added: '21.5.0'
   max_http_size:
     description:
-    - Specify delivery size limit for the HTTP transport protocol (in bytes).
+      - Specify delivery size limit for the HTTP transport protocol (in bytes).
     type: int
     version_added: '21.5.0'
   max_smtp_size:
     description:
-    - Specify delivery size limit for the SMTP transport protocol (in bytes).
+      - Specify delivery size limit for the SMTP transport protocol (in bytes).
     type: int
     version_added: '21.5.0'
   private_data_removed:
     description:
-    - Specify the removal of customer-supplied data.
+      - Specify the removal of customer-supplied data.
     type: bool
     version_added: '21.5.0'
   local_collection_enabled:
     description:
-    - Specify whether collection of AutoSupport data when the AutoSupport daemon is disabled.
+      - Specify whether collection of AutoSupport data when the AutoSupport daemon is disabled.
     type: bool
     version_added: '21.5.0'
   ondemand_enabled:
     description:
-    - Specify whether the AutoSupport OnDemand Download feature is enabled.
+      - Specify whether the AutoSupport OnDemand Download feature is enabled.
     type: bool
     version_added: '21.5.0'
   validate_digital_certificate:
     description:
-    - When set to true each node will validate the digital certificates that it receives.
+      - When set to true each node will validate the digital certificates that it receives.
     type: bool
     version_added: '21.5.0'
     """
@@ -184,10 +184,10 @@ from ansible.module_utils._text import to_native
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
 from ansible_collections.netapp.ontap.plugins.module_utils.netapp_module import NetAppModule
 from ansible_collections.netapp.ontap.plugins.module_utils.netapp import OntapRestAPI
-import ansible_collections.netapp.ontap.plugins.module_utils.rest_response_helpers as rrh
+from ansible_collections.netapp.ontap.plugins.module_utils import rest_generic
 
 
-class NetAppONTAPasup():
+class NetAppONTAPasup:
     """Class with autosupport methods"""
 
     def __init__(self):
@@ -274,63 +274,64 @@ class NetAppONTAPasup():
         get current autosupport details
         :return: dict()
         """
+        asup_info = {}
         if self.use_rest:
-            api = "/private/cli/system/node/autosupport"
+            api = "private/cli/system/node/autosupport"
             query = {
                 'node': self.parameters['node_name'],
                 'fields': 'state,node,transport,noteto,url,support,mail-hosts,from,partner-address,to,proxy-url,hostname-subj,nht,perf,retry-count,\
 reminder,max-http-size,max-smtp-size,remove-private-data,ondemand-server-url,support,reminder,ondemand-state,local-collection,validate-digital-certificate'
             }
-            message, error = self.rest_api.get(api, query)
-            records, error = rrh.check_for_0_or_more_records(api, message, error)
+            record, error = rest_generic.get_one_record(self.rest_api, api, query)
 
             if error:
-                self.module.fail_json(msg=error)
+                self.module.fail_json(msg='Error fetching info: %s' % error)
 
-            asup_info = {}
-            for param in ('transport', 'support', 'mail_hosts', 'proxy_url', 'retry_count',
+            for param in ('transport', 'mail_hosts', 'proxy_url', 'retry_count',
                           'max_http_size', 'max_smtp_size', 'noteto', 'validate_digital_certificate'):
-                if param in records[0]:
-                    asup_info[param] = records[0][param]
-
-            asup_info['node_name'] = records[0]['node'] if 'node' in records[0] else ""
-            asup_info['post_url'] = records[0]['url'] if 'url' in records[0] else ""
-            asup_info['from_address'] = records[0]['from'] if 'from' in records[0] else ""
-            asup_info['to_addresses'] = records[0]['to'] if 'to' in records[0] else list()
-            asup_info['hostname_in_subject'] = records[0]['hostname_subj'] if 'hostname_subj' in records[0] else False
-            asup_info['nht_data_enabled'] = records[0]['nht'] if 'nht' in records[0] else False
-            asup_info['perf_data_enabled'] = records[0]['perf'] if 'perf' in records[0] else False
-            asup_info['reminder_enabled'] = records[0]['reminder'] if 'reminder' in records[0] else False
-            asup_info['private_data_removed'] = records[0]['remove_private_data'] if 'remove_private_data' in records[0] else False
-            asup_info['local_collection_enabled'] = records[0]['local_collection'] if 'local_collection' in records[0] else False
-            asup_info['ondemand_enabled'] = records[0]['ondemand_state'] if 'ondemand_state' in records[0] else False
-            asup_info['service_state'] = 'started' if records[0]['state'] else 'stopped'
-
-            return asup_info
+                if param in record:
+                    asup_info[param] = record[param]
 
+            asup_info['support'] = record['support'] in ['enable', True]
+            asup_info['node_name'] = record['node'] if 'node' in record else ""
+            asup_info['post_url'] = record['url'] if 'url' in record else ""
+            asup_info['from_address'] = record['from'] if 'from' in record else ""
+            asup_info['to_addresses'] = record['to'] if 'to' in record else list()
+            asup_info['hostname_in_subject'] = record['hostname_subj'] if 'hostname_subj' in record else False
+            asup_info['nht_data_enabled'] = record['nht'] if 'nht' in record else False
+            asup_info['perf_data_enabled'] = record['perf'] if 'perf' in record else False
+            asup_info['reminder_enabled'] = record['reminder'] if 'reminder' in record else False
+            asup_info['private_data_removed'] = record['remove_private_data'] if 'remove_private_data' in record else False
+            asup_info['local_collection_enabled'] = record['local_collection'] if 'local_collection' in record else False
+            asup_info['ondemand_enabled'] = record['ondemand_state'] in ['enable', True] if 'ondemand_state' in record else False
+            asup_info['service_state'] = 'started' if record['state'] in ['enable', True] else 'stopped'
+            asup_info['partner_addresses'] = record['partner_address'] if 'partner_address' in record else list()
         else:
             asup_details = netapp_utils.zapi.NaElement('autosupport-config-get')
             asup_details.add_new_child('node-name', self.parameters['node_name'])
-            asup_info = dict()
             try:
                 result = self.server.invoke_successfully(asup_details, enable_tunneling=True)
             except netapp_utils.zapi.NaApiError as error:
-                self.module.fail_json(msg='%s' % to_native(error), exception=traceback.format_exc())
+                self.module.fail_json(msg='Error fetching info: %s' % to_native(error), exception=traceback.format_exc())
             # zapi invoke successful
             asup_attr_info = result.get_child_by_name('attributes').get_child_by_name('autosupport-config-info')
             asup_info['service_state'] = 'started' if asup_attr_info['is-enabled'] == 'true' else 'stopped'
             for item_key, zapi_key in self.na_helper.zapi_string_keys.items():
-                value = asup_attr_info[zapi_key]
+                value = asup_attr_info.get_child_content(zapi_key)
                 asup_info[item_key] = value if value is not None else ""
             for item_key, zapi_key in self.na_helper.zapi_int_keys.items():
-                asup_info[item_key] = self.na_helper.get_value_for_int(from_zapi=True, value=asup_attr_info[zapi_key])
+                value = asup_attr_info.get_child_content(zapi_key)
+                if value is not None:
+                    asup_info[item_key] = self.na_helper.get_value_for_int(from_zapi=True, value=value)
             for item_key, zapi_key in self.na_helper.zapi_bool_keys.items():
-                asup_info[item_key] = self.na_helper.get_value_for_bool(from_zapi=True, value=asup_attr_info[zapi_key])
+                value = asup_attr_info.get_child_content(zapi_key)
+                if value is not None:
+                    asup_info[item_key] = self.na_helper.get_value_for_bool(from_zapi=True, value=value)
             for item_key, zapi_key in self.na_helper.zapi_list_keys.items():
                 parent, dummy = zapi_key
                 asup_info[item_key] = self.na_helper.get_value_for_list(from_zapi=True, zapi_parent=asup_attr_info.get_child_by_name(parent))
 
-            return asup_info
+        return asup_info
 
     def modify_autosupport_config(self, modify):
         """
@@ -339,12 +340,12 @@ reminder,max-http-size,max-smtp-size,remove-private-data,ondemand-server-url,sup
         """
 
         if self.use_rest:
-            api = "/private/cli/system/node/autosupport"
+            api = "private/cli/system/node/autosupport"
             query = {
                 'node': self.parameters['node_name']
             }
             if 'service_state' in modify:
-                modify['state'] = True if modify['service_state'] == 'started' else False
+                modify['state'] = modify['service_state'] == 'started'
                 del modify['service_state']
 
             if 'post_url' in modify:
@@ -367,11 +368,13 @@ reminder,max-http-size,max-smtp-size,remove-private-data,ondemand-server-url,sup
                 modify['local_collection'] = modify.pop('local_collection_enabled')
             if 'ondemand_enabled' in modify:
                 modify['ondemand_state'] = modify.pop('ondemand_enabled')
+            if 'partner_addresses' in modify:
+                modify['partner_address'] = modify.pop('partner_addresses')
 
-            dummy, error = self.rest_api.patch(api, modify, query)
+            dummy, error = rest_generic.patch_async(self.rest_api, api, None, modify, query)
 
             if error:
-                self.module.fail_json(msg=error)
+                self.module.fail_json(msg='Error modifying asup: %s' % error)
         else:
             asup_details = {'node-name': self.parameters['node_name']}
             if modify.get('service_state'):
@@ -423,23 +426,19 @@ reminder,max-http-size,max-smtp-size,remove-private-data,ondemand-server-url,sup
                 sanitized_modify['proxy_url'] = "%s:XXXXXXXX@%s" % user_url_m
         return sanitized_modify
 
-    def ems_log_event(self):
-        results = netapp_utils.get_cserver(self.server)
-        cserver = netapp_utils.setup_na_ontap_zapi(module=self.module, vserver=results)
-        netapp_utils.ems_log_event("na_ontap_autosupport", cserver)
-
     def apply(self):
         """
         Apply action to autosupport
         """
         if not self.use_rest:
-            self.ems_log_event()
+            netapp_utils.ems_log_event_cserver("na_ontap_autosupport", self.server, self.module)
         current = self.get_autosupport_config()
         modify = self.na_helper.get_modified_attributes(current, self.parameters)
         sanitized_modify = self.idempotency_check(current, modify)
         if self.na_helper.changed and not self.module.check_mode:
             self.modify_autosupport_config(modify)
-        self.module.exit_json(changed=self.na_helper.changed, modify=sanitized_modify, current=current)
+        result = netapp_utils.generate_result(self.na_helper.changed, modify=sanitized_modify)
+        self.module.exit_json(**result)
 
 
 def main():
