@@ -164,11 +164,7 @@ class NetAppOntapEfficiencyPolicy(object):
 
         if self.use_rest and not self.rest_api.meets_rest_minimum_version(self.use_rest, 9, 8, 0):
             msg = 'REST requires ONTAP 9.8 or later for efficiency_policy APIs.'
-            if self.parameters['use_rest'].lower() == 'always':
-                self.module.fail_json(msg='Error: %s' % msg)
-            if self.parameters['use_rest'].lower() == 'auto':
-                self.module.warn('Falling back to ZAPI: %s' % msg)
-                self.use_rest = False
+            self.use_rest = self.na_helper.fall_back_to_zapi(self.module, msg, self.parameters)
 
         if self.parameters.get('policy_type') and self.parameters['state'] == 'present':
             if self.parameters['policy_type'] == 'threshold':
@@ -394,6 +390,7 @@ class NetAppOntapEfficiencyPolicy(object):
         if not self.use_rest:
             netapp_utils.ems_log_event("na_ontap_efficiency_policy", self.server)
         current = self.get_efficiency_policy()
+        modify = None
         cd_action = self.na_helper.get_cd_action(current, self.parameters)
         if cd_action is None and self.parameters['state'] == 'present':
             modify = self.na_helper.get_modified_attributes(current, self.parameters)
@@ -406,7 +403,8 @@ class NetAppOntapEfficiencyPolicy(object):
                 self.delete_efficiency_policy()
             elif modify:
                 self.modify_efficiency_policy(modify)
-        self.module.exit_json(changed=self.na_helper.changed)
+        result = netapp_utils.generate_result(self.na_helper.changed, cd_action, modify)
+        self.module.exit_json(**result)
 
 
 def main():
