@@ -33,24 +33,37 @@ def patch_request_and_invoke(request):
         print('entering patch_request_and_invoke fixture for', request.function)
     function_name = request.function.__name__
 
-    with patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request') as mock_send_request:
-        mock_send_request.side_effect = partial(_mock_netapp_send_request, function_name)
-        if netapp_utils.has_netapp_lib():
-            with patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapZAPICx.invoke_elem') as mock_invoke_elem:
-                mock_invoke_elem.side_effect = partial(_mock_netapp_invoke_elem, function_name)
-                yield mock_send_request, mock_invoke_elem
-        else:
-            yield mock_send_request
+    with patch('time.sleep') as mock_time_sleep:
+        mock_time_sleep.side_effect = partial(_mock_time_sleep, function_name)
+        with patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request') as mock_send_request:
+            mock_send_request.side_effect = partial(_mock_netapp_send_request, function_name)
+            if netapp_utils.has_netapp_lib():
+                with patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapZAPICx.invoke_elem') as mock_invoke_elem:
+                    mock_invoke_elem.side_effect = partial(_mock_netapp_invoke_elem, function_name)
+                    yield mock_send_request, mock_invoke_elem
+            else:
+                yield mock_send_request
 
     # This part is executed after the test completes
     _patch_request_and_invoke_exit_checks(function_name)
 
 
 def register_responses(responses, function_name=None):
+    ''' When patching, the pytest request identifies the UT test function
+        if the registration is happening in a helper function, function_name needs to identify the calling test function
+        EG:
+        test_me():
+            for x in range:
+                check_something()
+        if the registration happens in check_something, function_name needs to be set to test_me (as a string)
+    '''
     caller = inspect.currentframe().f_back.f_code.co_name
-    if function_name is not None and function_name != caller:
+    if DEBUG:
+        print('register_responses - caller:', caller, 'function_name:', function_name)
+    if function_name is not None and function_name != caller and (caller.startswith('test') or not function_name.startswith('test')):
         raise KeyError('inspect reported a different name: %s, received: %s' % (caller, function_name))
-    function_name = caller
+    if function_name is None:
+        function_name = caller
     fixed_records = []
     for record in responses:
         try:
@@ -166,11 +179,12 @@ class MockCalls:
                 return False
         return True
 
-    def _record_rest_request(self, method, api, params, json, headers):
+    def _record_rest_request(self, method, api, params, json, headers, files):
         record = {
             'params': params,
             'json': json,
-            'headers': headers
+            'headers': headers,
+            'files': files,
         }
         self._record_request(method, api, record)
 
@@ -218,6 +232,8 @@ def _get_response(function, method, api):
         print_requests(function)
         raise KeyError('function %s received an unexpected call %s %s, expecting %s %s' % (function, method, api, expected_method, expected_api))
     _RESPONSES[function].pop(0)
+    if isinstance(response, Exception):
+        raise response
     # some modules modify the record in place - keep the original intact
     return deepcopy(response)
 
@@ -228,23 +244,29 @@ def _get_or_create_mock_record(function_name):
     return _REQUESTS[function_name]
 
 
-def _mock_netapp_send_request(function_name, method, api, params, json=None, headers=None):
+def _mock_netapp_send_request(function_name, method, api, params, json=None, headers=None, files=None):
     if DEBUG:
-        print('Inside mock_netapp_send_request')
+        print('Inside _mock_netapp_send_request')
     mock_calls = _get_or_create_mock_record(function_name)
-    mock_calls._record_rest_request(method, api, params, json, headers)
+    mock_calls._record_rest_request(method, api, params, json, headers, files)
     return mock_calls._get_response(method, api)
 
 
 def _mock_netapp_invoke_elem(function_name, na_element, enable_tunneling=False):
     if DEBUG:
-        print('Inside mock_netapp_invoke_elem')
+        print('Inside _mock_netapp_invoke_elem')
     zapi = na_element.get_name()
     mock_calls = _get_or_create_mock_record(function_name)
     mock_calls._record_zapi_request(zapi, na_element, enable_tunneling)
     return mock_calls._get_response('ZAPI', zapi)
 
 
+def _mock_time_sleep(function_name, duration):
+    if DEBUG:
+        print('Inside _mock_time_sleep for %s' % function_name)
+    raise KeyError("time.sleep(%s) was called - add: @patch('time.sleep')" % duration)
+
+
 def _patch_request_and_invoke_exit_checks(function_name):
     # action to be performed afther a test is complete
     if DEBUG:
