@@ -41,7 +41,7 @@ __metaclass__ = type
 DOCUMENTATION = """
     lookup: nb_lookup
     author: Chris Mills (@cpmills1975)
-    version_added: "2.9"
+    version_added: "0.1.0"
     short_description: Queries and returns elements from NetBox
     description:
         - Queries NetBox via its API to return virtually any information
@@ -169,6 +169,7 @@ def get_endpoint(netbox, term):
         "circuits": {"endpoint": netbox.circuits.circuits},
         "circuit-providers": {"endpoint": netbox.circuits.providers},
         "cables": {"endpoint": netbox.dcim.cables},
+        "cable-terminations": {"endpoint": netbox.dcim.cable_terminations},
         "cluster-groups": {"endpoint": netbox.virtualization.cluster_groups},
         "cluster-types": {"endpoint": netbox.virtualization.cluster_types},
         "clusters": {"endpoint": netbox.virtualization.clusters},
@@ -206,12 +207,20 @@ def get_endpoint(netbox, term):
         "interface-templates": {"endpoint": netbox.dcim.interface_templates},
         "interfaces": {"endpoint": netbox.dcim.interfaces},
         "inventory-items": {"endpoint": netbox.dcim.inventory_items},
+        "inventory-item-roles": {"endpoint": netbox.dcim.inventory_item_roles},
+        "inventory-item-templates": {"endpoint": netbox.dcim.inventory_item_templates},
         "ip-addresses": {"endpoint": netbox.ipam.ip_addresses},
         "ip-ranges": {"endpoint": netbox.ipam.ip_ranges},
         "job-results": {"endpoint": netbox.extras.job_results},
         "journal-entries": {"endpoint": netbox.extras.journal_entries},
         "locations": {"endpoint": netbox.dcim.locations},
+        "l2vpn-terminations": {"endpoint": netbox.ipam.l2vpn_terminations},
+        "l2vpns": {"endpoint": netbox.ipam.l2vpns},
         "manufacturers": {"endpoint": netbox.dcim.manufacturers},
+        "module-bays": {"endpoint": netbox.dcim.module_bays},
+        "module-bay-templates": {"endpoint": netbox.dcim.module_bay_templates},
+        "module-bay-types": {"endpoint": netbox.dcim.module_bay_types},
+        "modules": {"endpoint": netbox.dcim.modules},
         "object-changes": {"endpoint": netbox.extras.object_changes},
         "permissions": {"endpoint": netbox.users.permissions},
         "platforms": {"endpoint": netbox.dcim.platforms},
@@ -239,6 +248,7 @@ def get_endpoint(netbox, term):
         "secret-roles": {"endpoint": netbox.secrets.secret_roles},
         "secrets": {"endpoint": netbox.secrets.secrets},
         "services": {"endpoint": netbox.ipam.services},
+        "service-templates": {"endpoint": netbox.ipam.service_templates},
         "site-groups": {"endpoint": netbox.dcim.site_groups},
         "sites": {"endpoint": netbox.dcim.sites},
         "tags": {"endpoint": netbox.extras.tags},
@@ -256,6 +266,29 @@ def get_endpoint(netbox, term):
         "webhooks": {"endpoint": netbox.extras.webhooks},
     }
 
+    major, minor, patch = map(int, pynetbox.__version__.split("."))
+
+    if major >= 6 and minor >= 4 and patch >= 0:
+        netbox_endpoint_map["wireless-lan-groups"] = {
+            "endpoint": netbox.wireless.wireless_lan_groups
+        }
+        netbox_endpoint_map["wireless-lan-groups"] = {
+            "endpoint": netbox.wireless.wireless_lan_groups
+        }
+        netbox_endpoint_map["wireless-lans"] = {
+            "endpoint": netbox.wireless.wireless_lans
+        }
+        netbox_endpoint_map["wireless-links"] = {
+            "endpoint": netbox.wireless.wireless_links
+        }
+
+    else:
+        if "wireless" in term:
+            Display().v(
+                "pynetbox version %d.%d.%d does not support wireless app; please update to v6.4.0 or newer."
+                % (major, minor, patch)
+            )
+
     return netbox_endpoint_map[term]["endpoint"]
 
 
@@ -399,7 +432,7 @@ class LookupModule(LookupBase):
                     )
 
             Display().vvvv(
-                u"NetBox lookup for %s to %s using token %s filter %s"
+                "NetBox lookup for %s to %s using token %s filter %s"
                 % (term, netbox_api_endpoint, netbox_api_token, netbox_api_filter)
             )
 
@@ -408,7 +441,7 @@ class LookupModule(LookupBase):
 
                 if "id" in filter:
                     Display().vvvv(
-                        u"Filter is: %s and includes id, will use .get instead of .filter"
+                        "Filter is: %s and includes id, will use .get instead of .filter"
                         % (filter)
                     )
                     try:
