@@ -23,8 +23,8 @@ short_description:  Manage network interfaces in a Pure Storage FlashArray
 version_added: '1.0.0'
 description:
     - This module manages the physical and virtual network interfaces on a Pure Storage FlashArray.
-    - To manage VLAN interfaces use the I(purefa_vlan) module.
-    - To manage network subnets use the I(purefa_subnet) module.
+    - To manage VLAN interfaces use the I(purestorage.flasharray.purefa_vlan) module.
+    - To manage network subnets use the I(purestorage.flasharray.purefa_subnet) module.
     - To remove an IP address from a non-management port use 0.0.0.0/0
 author: Pure Storage Ansible Team (@sdodsley) <pure-ansible-team@purestorage.com>
 options:
@@ -57,13 +57,23 @@ options:
     required: false
     default: 1500
     type: int
+  servicelist:
+    description:
+      - Assigns the specified (comma-separated) service list to one or more specified interfaces.
+      - Replaces the previous service list.
+      - Supported service lists depend on whether the network interface is Ethernet or Fibre Channel.
+      - Note that I(system) is only valid for Cloud Block Store.
+    elements: str
+    type: list
+    choices: [ "replication", "management", "ds", "file", "iscsi", "scsi-fc", "nvme-fc", "system"]
+    version_added: '1.15.0'
 extends_documentation_fragment:
     - purestorage.flasharray.purestorage.fa
 """
 
 EXAMPLES = """
 - name: Configure and enable network interface ct0.eth8
-  purefa_network:
+  purestorage.flasharray.purefa_network:
     name: ct0.eth8
     gateway: 10.21.200.1
     address: "10.21.200.18/24"
@@ -73,26 +83,34 @@ EXAMPLES = """
     api_token: c6033033-fe69-2515-a9e8-966bb7fe4b40
 
 - name: Disable physical interface ct1.eth2
-  purefa_network:
+  purestorage.flasharray.purefa_network:
     name: ct1.eth2
     state: absent
     fa_url: 10.10.10.2
     api_token: c6033033-fe69-2515-a9e8-966bb7fe4b40
 
 - name: Enable virtual network interface vir0
-  purefa_network:
+  purestorage.flasharray.purefa_network:
     name: vir0
     state: present
     fa_url: 10.10.10.2
     api_token: c6033033-fe69-2515-a9e8-966bb7fe4b40
 
 - name: Remove an IP address from iSCSI interface ct0.eth4
-  purefa_network:
+  purestorage.flasharray.purefa_network:
     name: ct0.eth4
     address: 0.0.0.0/0
     gateway: 0.0.0.0
     fa_url: 10.10.10.2
     api_token: c6033033-fe69-2515-a9e8-966bb7fe4b40
+
+- name: Change service list for FC interface ct0.fc1
+  purestorage.flasharray.purefa_network:
+    name: ct0.fc1
+    servicelist:
+      - replication
+    fa_url: 10.10.10.2
+    api_token: c6033033-fe69-2515-a9e8-966bb7fe4b40
 """
 
 RETURN = """
@@ -122,6 +140,13 @@ from ansible_collections.purestorage.flasharray.plugins.module_utils.purefa impo
 FC_ENABLE_API = "2.4"
 
 
+def _is_cbs(array, is_cbs=False):
+    """Is the selected array a Cloud Block Store"""
+    model = array.get(controllers=True)[0]["model"]
+    is_cbs = bool("CBS" in model)
+    return is_cbs
+
+
 def _get_fc_interface(module, array):
     """Return FC Interface or None"""
     interface = {}
@@ -153,31 +178,52 @@ def _get_interface(module, array):
     return interface
 
 
-def update_fc_interface(module, array, interface):
+def update_fc_interface(module, array, interface, api_version):
     """Modify FC Interface settings"""
     changed = False
-    if not interface.enabled and module.params["state"] == "present":
+    if FC_ENABLE_API in api_version:
+        if not interface.enabled and module.params["state"] == "present":
+            changed = True
+            if not module.check_mode:
+                network = NetworkInterfacePatch(enabled=True, override_npiv_check=True)
+                res = array.patch_network_interfaces(
+                    names=[module.params["name"]], network=network
+                )
+                if res.status_code != 200:
+                    module.fail_json(
+                        msg="Failed to enable interface {0}.".format(
+                            module.params["name"]
+                        )
+                    )
+        if interface.enabled and module.params["state"] == "absent":
+            changed = True
+            if not module.check_mode:
+                network = NetworkInterfacePatch(enabled=False, override_npiv_check=True)
+                res = array.patch_network_interfaces(
+                    names=[module.params["name"]], network=network
+                )
+                if res.status_code != 200:
+                    module.fail_json(
+                        msg="Failed to disable interface {0}.".format(
+                            module.params["name"]
+                        )
+                    )
+    if module.params["servicelist"] and sorted(module.params["servicelist"]) != sorted(
+        interface.services
+    ):
         changed = True
         if not module.check_mode:
-            network = NetworkInterfacePatch(enabled=True, override_npiv_check=True)
+            network = NetworkInterfacePatch(services=module.params["servicelist"])
             res = array.patch_network_interfaces(
                 names=[module.params["name"]], network=network
             )
             if res.status_code != 200:
                 module.fail_json(
-                    msg="Failed to enable interface {0}.".format(module.params["name"])
-                )
-    if interface.enabled and module.params["state"] == "absent":
-        changed = True
-        if not module.check_mode:
-            network = NetworkInterfacePatch(enabled=False, override_npiv_check=True)
-            res = array.patch_network_interfaces(
-                names=[module.params["name"]], network=network
-            )
-            if res.status_code != 200:
-                module.fail_json(
-                    msg="Failed to disable interface {0}.".format(module.params["name"])
+                    msg="Failed to update interface service list {0}. Error: {1}".format(
+                        module.params["name"], res.errors[0].message
+                    )
                 )
+
     module.exit_json(changed=changed)
 
 
@@ -189,6 +235,7 @@ def update_interface(module, array, interface):
         "gateway": interface["gateway"],
         "address": interface["address"],
         "netmask": interface["netmask"],
+        "services": sorted(interface["services"]),
     }
     if not module.params["address"]:
         address = interface["address"]
@@ -204,6 +251,7 @@ def update_interface(module, array, interface):
             ]:
                 module.fail_json(msg="Gateway and subnet are not compatible.")
         address = str(module.params["address"].split("/", 1)[0])
+    ip_version = str(IPAddress(address).version)
     if not module.params["mtu"]:
         mtu = interface["mtu"]
     else:
@@ -227,6 +275,8 @@ def update_interface(module, array, interface):
         if module.params["gateway"] not in IPNetwork(full_addr):
             module.fail_json(msg="Gateway and subnet are not compatible.")
         gateway = module.params["gateway"]
+    if ip_version == "6":
+        netmask = str(IPAddress(netmask).netmask_bits())
     new_state = {
         "address": address,
         "mtu": mtu,
@@ -282,6 +332,32 @@ def update_interface(module, array, interface):
                 module.fail_json(
                     msg="Failed to disable interface {0}.".format(interface["name"])
                 )
+    if (
+        module.params["servicelist"]
+        and sorted(module.params["servicelist"]) != interface["services"]
+    ):
+        api_version = array._list_available_rest_versions()
+        if FC_ENABLE_API in api_version:
+            if HAS_PYPURECLIENT:
+                array = get_array(module)
+                changed = True
+                if not module.check_mode:
+                    network = NetworkInterfacePatch(
+                        services=module.params["servicelist"]
+                    )
+                    res = array.patch_network_interfaces(
+                        names=[module.params["name"]], network=network
+                    )
+                    if res.status_code != 200:
+                        module.fail_json(
+                            msg="Failed to update interface service list {0}. Error: {1}".format(
+                                module.params["name"], res.errors[0].message
+                            )
+                        )
+            else:
+                module.warn_json(
+                    "Servicelist not update as pypureclient module is required"
+                )
 
     module.exit_json(changed=changed)
 
@@ -295,6 +371,20 @@ def main():
             address=dict(type="str"),
             gateway=dict(type="str"),
             mtu=dict(type="int", default=1500),
+            servicelist=dict(
+                type="list",
+                elements="str",
+                choices=[
+                    "replication",
+                    "management",
+                    "ds",
+                    "file",
+                    "iscsi",
+                    "scsi-fc",
+                    "nvme-fc",
+                    "system",
+                ],
+            ),
         )
     )
 
@@ -305,21 +395,21 @@ def main():
 
     array = get_system(module)
     api_version = array._list_available_rest_versions()
+    if not _is_cbs(array):
+        if "system" in module.params["servicelist"]:
+            module.fail_json(
+                msg="Only Cloud Block Store supports the 'system' service type"
+            )
     if "." in module.params["name"]:
         if module.params["name"].split(".")[1][0].lower() == "f":
-            if FC_ENABLE_API in api_version:
-                if not HAS_PYPURECLIENT:
-                    module.fail_json(msg="pypureclient module is required")
-                array = get_array(module)
-                interface = _get_fc_interface(module, array)
-                if not interface:
-                    module.fail_json(msg="Invalid network interface specified.")
-                else:
-                    update_fc_interface(module, array, interface)
+            if not HAS_PYPURECLIENT:
+                module.fail_json(msg="pypureclient module is required")
+            array = get_array(module)
+            interface = _get_fc_interface(module, array)
+            if not interface:
+                module.fail_json(msg="Invalid network interface specified.")
             else:
-                module.warn(
-                    "Purity version does not support enabling/disabling FC ports"
-                )
+                update_fc_interface(module, array, interface, api_version)
         else:
             interface = _get_interface(module, array)
             if not interface:
