@@ -81,7 +81,7 @@ class TestVyosFirewallRulesModule(TestVyosModule):
         self.mock_execute_show_command.stop()
         self.mock_get_os_version.stop()
 
-    def load_fixtures(self, commands=None):
+    def load_fixtures(self, commands=None, filename=None):
         def load_from_file(*args, **kwargs):
             return load_fixture("vyos_firewall_rules_config.cfg")
 
